/*------------------------------------------------------------------------------
 bin/djv_view/app.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include "app.h"
#include <djv_base/file.h>
#include "prefs.h"
#include "prefs_dialog.h"
#include "cache.h"
#include "window.h"
#include "image_color.h"
#include "magnify.h"
#include "color_sample.h"
#include "histogram.h"
#include "image_info.h"
#include <FL/Fl.H>

namespace djv_view
{

App::App(int & argc, char ** argv) : ui::App("djv_view", argc, argv)
{
  DJV_DBG_OBJ("App::App");
  
  // Command line.
  
  if (image::App::exit() != app::EXIT_DEFAULT) return;

  List<Str> input = cmdln_list;
  
  // Initialize.
  
  valid(true);

  prefs::init();
  prefs_dialog::init();

  if (input.size())
  {
    for (uint i = 0; i < input.size(); ++i)
    {
      File file(input[i]);
      if (file.seq_valid()) file.type = file::SEQ;
    
      window::Widget * window = window::factory()->add();
      window->file_open(file);
      window->show();
    }
  }
  else
  {
    window::Widget * window = window::factory()->add();
    window->show();
  }
}

App::~App()
{
  DJV_DBG_OBJ("App::~App");
  
  window::factory_del();
  image_color::del();
  magnify::del();
  color_sample::del();
  histogram::del();
  image_info::del();
  cache::del();
  prefs::del();
}

namespace
{
const Str _help =
"\n"
"djv_view\n"
"--------\n"
"\n"
"Image and movie viewing application.\n"
"\n"
"Command Line\n"
"------------\n"
"\n"
"Usage:\n"
"\n"
"djv_view [image]... [option]...\n"
"\n"
"%1";
}

Str App::help() const
{
  return str::Format(_help).
    arg(ui::App::help());
}

}

int main(int argc, char ** argv)
{
  int r = 1;
  try { r = djv_view::App(argc, argv).exec(); }
  catch (djv::Error in) { djv::error::print(in); }
  return r;
}

