/*------------------------------------------------------------------------------
 bin/djv_view/cache.h

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_VIEW_CACHE_H
#define DJV_VIEW_CACHE_H

#include "base.h"

namespace djv_view {
namespace cache
{

//------------------------------------------------------------------------------
// Image Reference

class Ref
{
public:

  Ref(image::Image *, int key, int frame);
  
  // Value.
  
  image::Image * get() { return _image.get(); }

  // Key.
  
  void key(int);
  int key() const { return _key; }
  int frame() const { return _frame; }
  
  // Count.

  void ref() { ++_ref_count; }
  void ref_del();
  int ref_count() const { return _ref_count; }
  
private:

  std::auto_ptr<image::Image> _image;
  int _key, _frame;
  int _ref_count;
};

//------------------------------------------------------------------------------
// Cache

class Cache
{
public:

  Cache();
  ~Cache();

  // Value.
  
  Ref * create(image::Image *, int key, int frame);
  Ref * get(int key, int frame);
  void max(int megabytes);
  int  max() const;
  void del(int key);
  void del();

  // Debugging.
  
  void dbg();

private:

  void purge();
  
  List<Ref *> _ref_list;
  uint64_t _cache_max, _cache_size;
};

//------------------------------------------------------------------------------
// Defaults

const List<int> & size_default();

//------------------------------------------------------------------------------
// Labels

const List<Str> & label_size();

//------------------------------------------------------------------------------
// Global

Cache * global();

void del();

}
}

#endif

