/*------------------------------------------------------------------------------
 bin/djv_view/view.h
 
 Image View

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_VIEW_VIEW_H
#define DJV_VIEW_VIEW_H

#include <djv_ui/image.h>
#include "base.h"

namespace djv_view {
namespace view
{

//------------------------------------------------------------------------------
// Options

enum GRID { GRID_NONE, GRID_1x1, GRID_10x10, GRID_100x100 };

struct Hud_Info
{
  File file;
  image::Info image;
  image::Tag tag;
  
  struct Playback
  {
    Playback() : frame(0), speed_real(0.0), dropped_frames(false) {}

    int frame;
    image::time::Speed speed;
    double speed_real;
    bool dropped_frames;
    
  } playback;
};

//------------------------------------------------------------------------------
// Widget

class Widget : public ui::image::Widget
{
public:

  Widget();

  // View.
  
  void zoom(double);
  void zoom(double, const Vec2i & focus);
  double zoom() const { return ui::image::Widget::zoom(); }
  
  // Overlays.
  
  void grid(GRID);
  void grid_color(const image::Color &);

  void hud(bool);
  void hud_info(const Hud_Info &);
  void hud_foreground(const image::Color &);
  void hud_background(const image::Color &);
  
  // Widget.
  
  int handle(int);
  void draw();
  
  ui::Signal<const Vec2i &> pick_sig;
  ui::Signal<const File &> file_sig;

private:

  void draw_grid();
  void draw_hud();
  static void draw_text(const Str &, const Vec2i & position,
    const image::Color & foreground, const image::Color & background);
  
  Vec2i _view_tmp;
  GRID _grid;
  image::Color _grid_color;
  bool _hud;
  Hud_Info _hud_info;
  image::Color _hud_foreground, _hud_background;
  bool _inside;
  Vec2i _mouse, _mouse_td;
};

//------------------------------------------------------------------------------
// Labels

const List<Str> & label_grid();

}
}

#endif

