/*------------------------------------------------------------------------------
 bin/djv_view/window_playback.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include "window.h"
#include <djv_ui/layout_row.h>
#include <djv_ui/button.h>
#include <djv_ui/text.h>
#include <djv_ui/shuttle.h>
#include <djv_ui/menu.h>
#include <FL/Fl.H>

namespace djv_view {
namespace window
{

Playback_Group::Playback_Group() : _menu(0) {}

void Playback_Group::menu(ui::menu::Widget * in, playback::PLAYBACK playback,
  bool every_frame)
{
  in->menu_add("&Playback", 0, 0, 0, ui::menu::SUB_MENU);
  
  _menu_playback.clear();
  _menu_playback += in->menu_add(
    "&Stop", '.', _stop_callback, this,
    ui::menu::RADIO | (playback::STOP == playback ? ui::menu::VALUE : 0));
  _menu_playback += in->menu_add(
    "&Forward", '/', _forward_callback, this,
    ui::menu::RADIO | (playback::FORWARD == playback ? ui::menu::VALUE : 0));
  _menu_playback += in->menu_add(
    "&Reverse", ',', _reverse_callback, this,
    ui::menu::RADIO | (playback::REVERSE == playback ? ui::menu::VALUE : 0) |
    ui::menu::DIVIDER);

  in->menu_add("S&tart Frame", ';', _start_callback, this);
  in->menu_add("&End Frame", '\'', _end_callback, this);
  in->menu_add("&Next Frame", ']', _next_callback, this);
  in->menu_add("Next Frame X10", FL_SHIFT + ']', _next_x10_callback, this);
  in->menu_add("Next Frame X100", FL_ALT + FL_SHIFT + ']', _next_x100_callback,
    this);
  in->menu_add("&Previous Frame", '[', _prev_callback, this);
  in->menu_add("Previous Frame X10", FL_SHIFT + '[', _prev_x10_callback, this);
  in->menu_add("Previous Frame X100", FL_ALT + FL_SHIFT + '[',
    _prev_x100_callback, this, ui::menu::DIVIDER);
  
  in->menu_add("&In Point Mark", 'i', _in_mark_callback, this);
  in->menu_add("In Po&int Reset", FL_SHIFT + 'i', _in_reset_callback, this);
  in->menu_add("&Out Point Mark", 'o', _out_mark_callback, this);
  in->menu_add("O&ut Point Reset", FL_SHIFT + 'o', _out_reset_callback, this,
    ui::menu::DIVIDER);

  in->menu_add("Ever&y Frame", 0, _every_frame_callback, this,
    ui::menu::TOGGLE | (every_frame ? ui::menu::VALUE : 0));
  
  in->menu_end();
  
  _menu = in;
}

void Playback_Group::playbar(ui::group::Frame * in)
{
  // Playback widgets.

  _stop_widget = new ui::button::Tool("playback_stop");
  _stop_widget->type(ui::button::Tool::RADIO);
  _stop_widget->tooltip("Stop playback.\nKeyboard: Period.");

  _forward_widget = new ui::button::Tool("playback_forward");
  _forward_widget->type(ui::button::Tool::RADIO);
  _forward_widget->tooltip(
    "Forward playback.\nKeyboard shortcut: Forward Slash.");

  _reverse_widget = new ui::button::Tool("playback_reverse");
  _reverse_widget->type(ui::button::Tool::RADIO);
  _reverse_widget->tooltip("Reverse playback.\nKeyboard shortcut: Comma.");

  ui::button::radio_group(List<ui::button::Tool *>() << _stop_widget <<
    _forward_widget << _reverse_widget);

  ui::shuttle::Widget * playback_shuttle = new ui::shuttle::Widget("playback_");
  playback_shuttle->tooltip("Playback shuttle.");

  // Frame widgets.
  
  _frame_widget = new playback::Frame_Widget;
  _frame_widget->tooltip("Current frame.");
  
  _frame_slider = new playback::Frame_Slider;
  _frame_slider->tooltip("Frame slider.");

  ui::shuttle::Widget * frame_shuttle = new ui::shuttle::Widget(
    "playback_frame_");
  frame_shuttle->tooltip("Frame shuttle.");

  _start_widget = new playback::Time_Widget;
  _start_widget->tooltip("Start frame.");
  ui::button::Tool * start_widget = new ui::button::Tool("playback_start");
  start_widget->tooltip("Start frame.\nKeyboard shortcut: Semicolon.");

  _end_widget = new playback::Time_Widget;
  _end_widget->tooltip("End frame.");
  ui::button::Tool * end_widget = new ui::button::Tool("playback_end");
  end_widget->tooltip("End frame.\nKeyboard shortcut: Right Quote.");
  
  ui::button::Tool * prev_widget = new ui::button::Tool("playback_prev");
  prev_widget->type(ui::button::Tool::REPEAT);
  prev_widget->tooltip("Previous frame.\nKeyboard shortcut: Left Bracket.");
  
  ui::button::Tool * next_widget = new ui::button::Tool("playback_next");
  next_widget->type(ui::button::Tool::REPEAT);
  next_widget->tooltip("Next frame.\nKeyboard shortcut: Right Bracket.");

  // In/out point widgets.
  
  ui::button::Tool * in_mark_widget = new ui::button::Tool("tool_start");
  in_mark_widget->tooltip("Mark in point.\nKeyboard shortcut: I.");
  
  ui::button::Tool * in_reset_widget = new ui::button::Tool("tool_x");
  in_reset_widget->tooltip("Reset in point.\nKeyboard shortcut: Shift + I.");
  
  ui::button::Tool * out_mark_widget = new ui::button::Tool("tool_end");
  out_mark_widget->tooltip("Mark out point.\nKeyboard shortcut: O.");
  
  ui::button::Tool * out_reset_widget = new ui::button::Tool("tool_x");
  out_reset_widget->tooltip("Reset out point.\nKeyboard shortcut: Shift + O.");
  
  // Speed widgets.
  
  _speed_widget = new playback::Speed_Widget;
  _speed_widget->tooltip("Playback speed.");
  
  _speed_real_widget = new playback::Speed_Display_Widget;
  _speed_real_widget->tooltip("Real playback speed.");
  
  _every_frame_widget = new ui::button::Tool("tool_lock");
  _every_frame_widget->type(ui::button::Tool::TOGGLE);
  _every_frame_widget->tooltip("Playback every frame.");

  // Layout.
  
  ui::layout::Horizontal * layout = new ui::layout::Horizontal(in);
  layout->spacing(0);
  layout->margin(0);
  
  ui::layout::Vertical * left_layout = new ui::layout::Vertical(layout);
  left_layout->margin(0);
  ui::layout::Horizontal * h_layout = new ui::layout::Horizontal(left_layout);
  h_layout->spacing(0);
  h_layout->margin(0);
  h_layout->add(_reverse_widget);
  h_layout->add(_stop_widget);
  h_layout->add(_forward_widget);
  h_layout->add(playback_shuttle);
  h_layout->add(start_widget);
  h_layout->add(prev_widget);
  h_layout->add(next_widget);
  h_layout->add(end_widget);
  h_layout->add(frame_shuttle);
  left_layout->add_stretch();
  h_layout = new ui::layout::Horizontal(left_layout);
  h_layout->add(_speed_widget);
  h_layout->add(_speed_real_widget);
  h_layout->add(_every_frame_widget);
  h_layout->add_stretch();
  
  ui::layout::Vertical * right_layout = new ui::layout::Vertical(layout);
  layout->stretch(right_layout);
  h_layout = new ui::layout::Horizontal(right_layout);
  h_layout->margin(0);
  h_layout->add(_frame_slider);
  h_layout->stretch(_frame_slider);
  right_layout->add_spacer(-1, true);
  h_layout = new ui::layout::Horizontal(right_layout);
  h_layout->margin(0);
  h_layout->add(_frame_widget);
  h_layout->add(in_mark_widget);
  h_layout->add(in_reset_widget);
  h_layout->add(_start_widget);
  h_layout->add_spacer(-1, true);
  h_layout->add(_end_widget);
  h_layout->add(out_reset_widget);
  h_layout->add(out_mark_widget);
  
  // Playback callbacks.
  
  _stop_widget->sig.set(this, stop_callback);
  _forward_widget->sig.set(this, forward_callback);
  _reverse_widget->sig.set(this, reverse_callback);
  
  playback_shuttle->sig.set(this, playback_shuttle_callback);
  playback_shuttle->value_sig.set(this, playback_shuttle_value_callback);
  
  // Frame callbacks.
  
  _frame_widget->sig.set(this, frame_callback);
  _frame_slider->sig.set(this, frame_callback);
  frame_shuttle->sig.set(this, frame_shuttle_callback);
  frame_shuttle->value_sig.set(this, frame_shuttle_value_callback);
  
  start_widget->sig.set(this, start_callback);
  end_widget->sig.set(this, end_callback);
  prev_widget->sig.set(this, prev_callback);
  next_widget->sig.set(this, next_callback);
  
  // In/out callbacks.
  
  _start_widget->mark_sig.set(this, in_callback);
  in_mark_widget->sig.set(this, in_mark_callback);
  in_reset_widget->sig.set(this, in_reset_callback);
  
  _end_widget->mark_sig.set(this, out_callback);
  out_mark_widget->sig.set(this, out_mark_callback);
  out_reset_widget->sig.set(this, out_reset_callback);
  
  // Speed callbacks.
  
  _speed_widget->sig.set(this, speed_callback);
  _every_frame_widget->sig.set(this, every_frame_callback);
}

void Playback_Group::set(playback::PLAYBACK in)
{
  switch (in)
  {
    case playback::STOP: _stop_widget->set(); break;
    case playback::FORWARD: _forward_widget->set(); break;
    case playback::REVERSE: _reverse_widget->set(); break;
  }
}

void Playback_Group::time(const image::Time & time, int in, int out)
{
  DJV_DBG_OBJ("Playback_Group::time");
  DJV_DBG_PRN("time = " << time.start << " " << time.end);
  DJV_DBG_PRN("in/out = " << in << " " << out);
  
  _frame_widget->range(time.start, time.end);
  _frame_slider->range(time.start, time.end);
  _frame_slider->in_out(in, out);

  _start_widget->range(time.start, time.end);
  _end_widget->range(time.start, time.end);
  if (time.start == in)
  {
    _start_widget->frame(time.start);
    _start_widget->mark(false);
  }
  else
  {
    _start_widget->frame(in);
    _start_widget->mark(true);
  }
  if (time.end == out)
  {
    _end_widget->frame(time.end);
    _end_widget->mark(false);
  }
  else
  {
    _end_widget->frame(out);
    _end_widget->mark(true);
  }
}

void Playback_Group::speed(const image::time::Speed & in, bool every_frame)
{
  _frame_widget->speed(in);
  _start_widget->speed(in);
  _end_widget->speed(in);
  _speed_widget->set(in);
  _every_frame_widget->set(every_frame);
}

void Playback_Group::speed_real(double in, bool dropped_frames)
{
  _speed_real_widget->set(in);
  _speed_real_widget->dropped_frames(dropped_frames);
}

void Playback_Group::speed_default(const image::time::Speed & in)
{
  _speed_widget->set_default(in);
}

void Playback_Group::playback_callback(playback::PLAYBACK in)
{ playback_sig.emit(in); }
void Playback_Group::stop_callback(bool) { playback_callback(playback::STOP); }
void Playback_Group::forward_callback(bool)
{ playback_callback(playback::FORWARD); }
void Playback_Group::reverse_callback(bool)
{ playback_callback(playback::REVERSE); }
void Playback_Group::_stop_callback() { stop_callback(true); }
void Playback_Group::_forward_callback() { forward_callback(true); }
void Playback_Group::_reverse_callback() { reverse_callback(true); }

void Playback_Group::playback_shuttle_callback(bool in)
{ playback_shuttle_sig.emit(in); }
void Playback_Group::playback_shuttle_value_callback(int in)
{ playback_shuttle_value_sig.emit(in); }

void Playback_Group::frame_callback(int in) { frame_sig.emit(in); }
void Playback_Group::frame_move_callback(playback::FRAME in)
{ frame_move_sig.emit(in); }

void Playback_Group::frame_shuttle_callback(bool in)
{ frame_shuttle_sig.emit(in); }
void Playback_Group::frame_shuttle_value_callback(int in)
{ frame_shuttle_value_sig.emit(in); }

void Playback_Group::start_callback(bool)
{ frame_move_callback(playback::START); }
void Playback_Group::_start_callback() { start_callback(true); }

void Playback_Group::end_callback(bool)
{ frame_move_callback(playback::END); }
void Playback_Group::_end_callback() { end_callback(true); }

void Playback_Group::prev_callback(bool)
{ frame_move_callback(playback::PREV); }
void Playback_Group::_prev_callback() { prev_callback(true); }
void Playback_Group::_prev_x10_callback()
{ frame_move_callback(playback::PREV_X10); }
void Playback_Group::_prev_x100_callback()
{ frame_move_callback(playback::PREV_X100); }

void Playback_Group::next_callback(bool)
{ frame_move_callback(playback::NEXT); }
void Playback_Group::_next_callback() { next_callback(true); }
void Playback_Group::_next_x10_callback()
{ frame_move_callback(playback::NEXT_X10); }
void Playback_Group::_next_x100_callback()
{ frame_move_callback(playback::NEXT_X100); }

void Playback_Group::frame(int in)
{
  _frame_widget->set(in);
  _frame_slider->set(in);
}

void Playback_Group::in_callback(int in) { in_sig.emit(in); }
void Playback_Group::in_mark_callback(bool) { in_mark_sig.emit(true); }
void Playback_Group::_in_mark_callback() { in_mark_callback(true); }
void Playback_Group::in_reset_callback(bool) { in_reset_sig.emit(true); }
void Playback_Group::_in_reset_callback() { in_reset_callback(true); }

void Playback_Group::out_callback(int in) { out_sig.emit(in); }
void Playback_Group::out_mark_callback(bool) { out_mark_sig.emit(true); }
void Playback_Group::_out_mark_callback() { out_mark_callback(true); }
void Playback_Group::out_reset_callback(bool) { out_reset_sig.emit(true); }
void Playback_Group::_out_reset_callback() { out_reset_callback(true); }

void Playback_Group::speed_callback(const image::time::Speed & in)
{ speed_sig.emit(in); }

void Playback_Group::every_frame_callback(bool in) { every_frame_sig.emit(in); }
void Playback_Group::_every_frame_callback()
{ every_frame_callback(_menu->value()); }

void Widget::playback(playback::PLAYBACK in)
{
  if (in == _playback_mode) return;
  
  DJV_DBG_OBJ("window::Widget::playback");
  DJV_DBG_PRN("in = " << in);
  
  _playback_mode = in;
  playback_update();
  playback_speed_update();
  menu_update();
}

void Widget::playback_callback(playback::PLAYBACK in) { playback(in); }

void Widget::playback_shuttle_callback(bool in)
{
  DJV_DBG_OBJ("window::Widget::playback_shuttle_callback");
  DJV_DBG_PRN("in = " << in);
  
  if (in)
  {
    playback(playback::STOP);
    _playback_shuttle = true;
    _playback_shuttle_speed = 0.0;
    _playback_timer_init = true;
  
    Fl::remove_idle(playback_timer_callback, this);
    Fl::add_idle(playback_timer_callback, this);
  }
  else
  {
    _playback_shuttle = false;
    _playback_dropped_frames = false;
  
    Fl::remove_idle(playback_timer_callback, this);
    
    playback_speed_update();
  }
}

void Widget::playback_shuttle_value_callback(int in)
{
  DJV_DBG_OBJ("window::Widget::playback_shuttle_value_callback");

  _playback_shuttle_speed = math::pow(double(math::abs(in)), 1.5) *
    (in >= 0 ? 1.0 : -1.0);
  //_playback_shuttle_speed = double(in);
  DJV_DBG_PRN("speed = " << _playback_shuttle_speed);
  
  _playback_timer_frame = uint64_t(_playback_timer.seconds() *
    _playback_shuttle_speed);
}

void Widget::playback_update()
{
  DJV_DBG_OBJ("window::Widget::playback_update");

  callbacks(false);
  
  _playback_group->set(_playback_mode);
  
  Fl::remove_idle(playback_timer_callback, this);
  //Fl::remove_timeout(playback_timer_callback, this);
  switch (_playback_mode)
  {
    case playback::REVERSE:
    case playback::FORWARD:
      _playback_timer_init = true;
      Fl::add_idle(playback_timer_callback, this);
      //Fl::add_timeout(1.0 / image::time::speed_to_float(_playback_speed),
      //  playback_timer_callback, this);
      break;
    case playback::STOP: break;
  }

  callbacks(true);
}

void Widget::playback_time(const image::Time & in)
{
  if (in == _playback_time) return;

  DJV_DBG_OBJ("window::Widget::time");
  DJV_DBG_PRN("in = " << in);
  
  _playback_time = in;
  _playback_in = _playback_time.start;
  _playback_out = _playback_time.end;
  _playback_speed_real = 0.0;
  
  playback_time_update();
  playback_frame_update();
  playback_speed_update();
}

void Widget::playback_time_update()
{
  callbacks(false);
  
  _playback_group->time(_playback_time, _playback_in, _playback_out);
  
  callbacks(true);
}

void Widget::playback_frame(int in)
{
  int tmp = 0;
  if (_playback_shuttle)
  {
    tmp = math::wrap(in, _playback_time.start, _playback_time.end);
  }
  else if (_playback_mode != playback::STOP)
  {
    tmp = math::wrap(in,
      math::max(_playback_time.start, _playback_in),
      math::min(_playback_time.end, _playback_out));
  }
  else
  {
    tmp = math::wrap(in, _playback_time.start, _playback_time.end);
  }
  if (tmp == _playback_frame) return;
  
  DJV_DBG_OBJ("window::Widget::playback_frame");
  DJV_DBG_PRN("in = " << in);
  
  _playback_frame = tmp;
  
  playback_frame_update();
  image_update();
}

void Widget::playback_frame_callback(int in)
{
  playback(playback::STOP);
  playback_frame(in);
}

void Widget::playback_frame_move_callback(playback::FRAME in)
{
  playback(playback::STOP);
  int frame = _playback_frame;
  switch (in)
  {
    case playback::START:
      frame = math::max(_playback_time.start, _playback_in);
      break;
    case playback::END:
      frame = math::min(_playback_time.end, _playback_out);
      break;
    case playback::PREV: frame -= 1; break;
    case playback::PREV_X10: frame -= 10; break;
    case playback::PREV_X100: frame -= 100; break;
    case playback::NEXT: frame += 1; break;
    case playback::NEXT_X10: frame += 10; break;
    case playback::NEXT_X100: frame += 100; break;
  }
  playback_frame(frame);
}

void Widget::playback_frame_shuttle_callback(bool)
{
  playback(playback::STOP);
  _playback_frame_tmp = _playback_frame;
}

void Widget::playback_frame_shuttle_value_callback(int in)
{
  playback_frame(_playback_frame_tmp + in);
}

void Widget::playback_frame_update()
{
  callbacks(false);
  
  _playback_group->frame(_playback_frame);
  
  callbacks(true);
}

void Widget::playback_in(int in)
{
  if (in == _playback_in) return;
  _playback_in = in;
  playback_frame(_playback_frame);
  playback_time_update();
  menu_update();
}

void Widget::playback_in_callback(int in) { playback_in(in); }
void Widget::playback_in_mark_callback(bool) { playback_in(_playback_frame); }
void Widget::playback_in_reset_callback(bool)
{ playback_in(_playback_time.start); }

void Widget::playback_out(int in)
{
  if (in == _playback_out) return;
  _playback_out = in;
  playback_frame(_playback_frame);
  playback_time_update();
  menu_update();
}

void Widget::playback_out_callback(int in) { playback_out(in); }
void Widget::playback_out_mark_callback(bool) { playback_out(_playback_frame); }
void Widget::playback_out_reset_callback(bool)
{ playback_out(_playback_time.end); }

void Widget::playback_speed_callback(const image::time::Speed & in)
{
  DJV_DBG_OBJ("window::Widget::playback_speed_callback");
  DJV_DBG_PRN("in = " << in);
  
  _playback_time.speed = in;
  _playback_speed_real = 0.0;
  _playback_dropped_frames = false;

  view_overlay_update();
  playback_update();
  playback_speed_update();
  menu_update();
}

void Widget::playback_every_frame_callback(bool in)
{
  DJV_DBG_OBJ("window::Widget::playback_every_frame_callback");
  DJV_DBG_PRN("in = " << in);
  
  _playback_every_frame = in;
  view_overlay_update();
  playback_speed_update();
  menu_update();
}

void Widget::playback_speed_update()
{
  DJV_DBG_OBJ("window::Widget::speed_update");
  
  callbacks(false);
  
  _playback_group->speed(_playback_time.speed, _playback_every_frame);
  _playback_group->speed_real(_playback_speed_real, _playback_dropped_frames);
  _playback_group->speed_default(_time.speed);
  
  callbacks(true);
}

void window::Widget::playback_timer_callback()
{
  DJV_DBG_OBJ("window::Widget::playback_timer_callback");
  DJV_DBG_PRN("playback = " << _playback_mode);
  DJV_DBG_PRN("frame = " << _playback_frame);
  DJV_DBG_PRN("every frame = " << _playback_every_frame);

  if (_playback_timer_init)
  {
    DJV_DBG_PRN("init");
    _playback_timer.start();
    _playback_timer_frame = 0;
    _playback_speed_timer = _playback_timer;
    _playback_speed_counter = 0;
    _playback_dropped_frames = false;
    _playback_dropped_frames_tmp = false;
    _playback_timer_init = false;
  }
  else
  {
    _playback_timer.check();
  }
  
  const double speed = _playback_shuttle ? _playback_shuttle_speed :
    image::time::speed_to_float(_playback_time.speed);
  DJV_DBG_PRN("speed = " << speed);

  // XXX Overflow?

  const uint64_t absolute_frame = uint64_t(_playback_timer.seconds() * speed);
  int inc = int(absolute_frame - _playback_timer_frame);
  DJV_DBG_PRN("absolute frame = " << uint(absolute_frame));
  DJV_DBG_PRN("inc = " << inc);
  if (! inc) return;

  // Calculate current frame.

  _playback_timer_frame = absolute_frame;
  if (playback::REVERSE == _playback_mode) inc = -inc;
  const bool every_frame = _playback_every_frame && ! _playback_shuttle;
  if (every_frame) inc = inc > 0 ? 1 : (inc < 0 ? -1 : 0);
  else _playback_dropped_frames_tmp |= inc > 1;
  _playback_speed_counter += abs(inc);
  
  // Calculate real playback speed.

  _playback_speed_timer.check();
  if (_playback_speed_timer.seconds() >= 1.0)
  {
    _playback_speed_real = _playback_speed_counter /
      _playback_speed_timer.seconds();
    _playback_speed_timer.start();

    _playback_dropped_frames = _playback_dropped_frames_tmp;
    _playback_dropped_frames_tmp = false;
    _playback_speed_counter = 0;

    _playback_group->speed_real(_playback_speed_real,
      every_frame ? false : _playback_dropped_frames);
  }

  playback_frame(_playback_frame + inc);
  
  //Fl::repeat_timeout(1.0 / image::time::speed_to_float(_playback_speed),
  //  playback_timer_callback, this);
}

}
}

