/*------------------------------------------------------------------------------
 bin/djv_view/window_view.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include "window.h"
#include <djv_ui/style.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/label.h>
#include <djv_ui/button.h>
#include <djv_ui/num.h>
#include <djv_ui/menu.h>
#include "prefs.h"

namespace djv_view {
namespace window
{

View_Group::View_Group() : _menu(0) {}

void View_Group::zoom(double in)
{
  _zoom_label->set(str::Format("%1%").arg(int(in * 100)));
}

void View_Group::menu(ui::menu::Widget * in, view::GRID grid, bool hud)
{
  in->menu_add("&View", 0, 0, 0, ui::menu::SUB_MENU);
  
  in->menu_add("&Left", FL_Left, _left_callback, this);
  in->menu_add("&Right", FL_Right, _right_callback, this);
  in->menu_add("&Up", FL_Up, _up_callback, this);
  in->menu_add("&Down", FL_Down, _down_callback, this);
  in->menu_add("&Center", FL_End, _center_callback, this, ui::menu::DIVIDER);
  
  in->menu_add("Zoom &In", '=', _zoom_in_callback, this);
  in->menu_add("Zoom &Out", '-', _zoom_out_callback, this);
  in->menu_add("Zoom R&eset", '0', _zoom_reset_callback, this,
    ui::menu::DIVIDER);
  
  in->menu_add("Rese&t", FL_Home, _reset_callback, this);
  
  in->menu_add("&Fit", FL_BackSpace, _fit_callback, this, ui::menu::DIVIDER);
  
  in->menu_add("&Grid", 0, 0, 0, ui::menu::SUB_MENU);
  _menu_grid.clear();
  List<Str> tmp = view::label_grid();
  for (uint i = 0; i < tmp.size(); ++i)
    _menu_grid += in->menu_add(
      ui::menu::list_name(tmp[i], i), 0,
      _grid_callback, this,
      ui::menu::RADIO | (int(i) == grid ? ui::menu::VALUE : 0));
  in->menu_end();
  
  in->menu_add("&HUD", 'h', _hud_callback, this,
    ui::menu::TOGGLE | (hud ? ui::menu::VALUE : 0));
    
  in->menu_end();
  
  _menu = in;
}

void View_Group::toolbar(ui::layout::Horizontal * in)
{
  ui::button::Tool * zoom_in_widget = new ui::button::Tool("view_zoom_in");
  zoom_in_widget->type(ui::button::Tool::REPEAT);
  zoom_in_widget->tooltip("Zoom view in.\nKeyboard shortcut: Equal.");
  
  ui::button::Tool * zoom_out_widget = new ui::button::Tool("view_zoom_out");
  zoom_out_widget->type(ui::button::Tool::REPEAT);
  zoom_out_widget->tooltip("Zoom view out.\nKeyboard shortcut: Minus.");
  
  _zoom_label = new ui::label::Widget;
  _zoom_label->size_str("%000");
  _zoom_label->align(ui::layout::RIGHT, ui::layout::CENTER);
  _zoom_label->tooltip("View zoom.");
  ui::num::Inc * zoom_inc_widget = new ui::num::Inc;

  ui::button::Tool * zoom_reset_widget = new ui::button::Tool(
    "view_zoom_reset");
  zoom_reset_widget->tooltip("Reset zoom.\nKeyboard shortcut: 0.");
  
  ui::button::Tool * fit_widget = new ui::button::Tool("view_fit");
  fit_widget->tooltip("Fit view to image.\nKeyboard shortcut: Backspace.");

  in->add(zoom_in_widget);
  in->add(zoom_out_widget);
  in->add_spacer(ui::style::global()->spacing());
  in->add(_zoom_label);
  in->add_spacer(ui::style::global()->spacing());
  in->add(zoom_inc_widget);
  in->add_spacer(ui::style::global()->spacing());
  in->add(zoom_reset_widget);
  in->add(fit_widget);
  
  zoom_in_widget->sig.set(this, zoom_in_callback);
  zoom_out_widget->sig.set(this, zoom_out_callback);
  zoom_inc_widget->inc_sig.set(this, zoom_inc_callback);
  zoom_inc_widget->dec_sig.set(this, zoom_dec_callback);
  zoom_reset_widget->sig.set(this, zoom_reset_callback);
  fit_widget->sig.set(this, fit_callback);
}

void View_Group::_left_callback() { move_sig.emit(Vec2i(10, 0)); }
void View_Group::_right_callback() { move_sig.emit(Vec2i(-10, 0)); }
void View_Group::_up_callback() { move_sig.emit(Vec2i(0, 10)); }
void View_Group::_down_callback() { move_sig.emit(Vec2i(0, -10)); }
void View_Group::_center_callback() { center_sig.emit(true); }

void View_Group::zoom_in_callback(bool) { zoom_sig.emit(2.0); }
void View_Group::zoom_out_callback(bool) { zoom_sig.emit(0.5); }
void View_Group::zoom_inc_callback(bool) { zoom_sig.emit(1.1); }
void View_Group::zoom_dec_callback(bool) { zoom_sig.emit(0.9); }
void View_Group::zoom_reset_callback(bool) { zoom_reset_sig.emit(true); }
void View_Group::_zoom_in_callback() { zoom_in_callback(true); }
void View_Group::_zoom_out_callback() { zoom_out_callback(true); }
void View_Group::_zoom_reset_callback() { zoom_reset_callback(true); }

void View_Group::_reset_callback() { reset_sig.emit(true); }

void View_Group::fit_callback(bool) { fit_sig.emit(true); }
void View_Group::_fit_callback() { fit_callback(true); }

void View_Group::_grid_callback()
{ grid_sig.emit(view::GRID(list::find(_menu->menu(), _menu_grid))); }
void View_Group::_hud_callback() { hud_sig.emit(_menu->value()); }

void Widget::view_callback(bool) { view_update(); }

void Widget::view_move_callback(const Vec2i & in)
{
  _view->view(_view->view() + in);
  _view->redraw();
}

void Widget::view_center_callback(bool)
{
  _view->view_center();
  _view->redraw();
}

void Widget::view_zoom_callback(double in)
{
  _view->zoom(_view->zoom() * in);
  _view->redraw();
}

void Widget::view_zoom_reset_callback(bool)
{
  _view->zoom(1.0);
  _view->redraw();
}

void Widget::view_reset_callback(bool)
{
  _view->view_zero();
  _view->redraw();
}

void Widget::view_fit_callback(bool)
{
  _view->view_fit();
  _view->redraw();
}

void Widget::view_update()
{
  callbacks(false);

  _view_group->zoom(_view->zoom());

  _image_group->pick(_image_pick, _view->zoom());
  
  _view->background(prefs::view()->background());
  
  callbacks(true);
}

void Widget::view_grid_callback(view::GRID in)
{
  _view_grid = in;
  view_overlay_update();
  menu_update();
  _view->redraw();
}

void Widget::view_grid_color_callback(const image::Color & in)
{
  _view->grid_color(in);
  _view->redraw();
}

void Widget::view_hud_callback(bool in)
{
  _view_hud = in;
  view_overlay_update();
  menu_update();
  _view->redraw();
}

void Widget::view_hud_foreground_callback(const image::Color & in)
{
  _view->hud_foreground(in);
  _view->redraw();
}

void Widget::view_hud_background_callback(const image::Color & in)
{
  _view->hud_background(in);
  _view->redraw();
}

void Widget::view_overlay_update()
{
  DJV_DBG_OBJ("window::Widget::view_overlay_update");
  
  callbacks(false);

  _view->grid(_view_grid);
  _view->grid_color(prefs::view()->grid_color());

  view::Hud_Info info;
  info.file = _file;
  if (_image_p)
  {
    info.image = _image_p->info();
    info.tag = _image_p->tag;
  }
  info.playback.frame = _playback_frame;
  info.playback.speed = _playback_time.speed;
  info.playback.speed_real = _playback_speed_real;
  info.playback.dropped_frames = _playback_dropped_frames;
  
  _view->hud(_view_hud);
  _view->hud_info(info);
  _view->hud_foreground(prefs::view()->hud_foreground());
  _view->hud_background(prefs::view()->hud_background());

  callbacks(true);
}

void Widget::view_pick_callback(const Vec2i & in)
{
  DJV_DBG_OBJ("window::Widget::view_pick_callback");
  DJV_DBG_PRN("in = " << in);
  
  _image_pick = in - _view->view();
  image_pick_update();
}

void Widget::view_background_callback(const image::Color & in)
{
  _view->background(in);
  _view->redraw();
}

}
}

