/*------------------------------------------------------------------------------
 bin/djv_vlut/widget.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include "widget.h"
#include <djv_ui/layout_row.h>
#include <djv_ui/group.h>
#include <djv_ui/label.h>
#include <djv_ui/button.h>
#include <djv_ui/num_float.h>
#include <djv_ui/file.h>

namespace djv_vlut {
namespace widget
{

Widget::Widget()
{
  // Widgets.
  
  ui::group::Box * lut_group = new ui::group::Box("Lookup Table");
  _lut = new ui::file::Widget;
  
  ui::group::Box * color_group = new ui::group::Box("Color");
  _brightness = new ui::num::Float_Slider;
  _brightness->default_value(Color().color.brightness);
  _brightness->range(0.0, 4.0);
  _brightness->label(new ui::label::Widget("Brightness:"));
  _contrast = new ui::num::Float_Slider;
  _contrast->default_value(Color().color.contrast);
  _contrast->range(0.0, 4.0);
  _contrast->label(new ui::label::Widget("Contrast:"));
  
  ui::group::Box * levels_group = new ui::group::Box("Levels");
  _levels_in[0] = new ui::num::Float_Slider;
  _levels_in[0]->default_value(Color().levels.in[0]);
  _levels_in[0]->range(0.0, 1.0);
  _levels_in[0]->label(new ui::label::Widget("Input:"));
  _levels_in[1] = new ui::num::Float_Slider;
  _levels_in[1]->default_value(Color().levels.in[1]);
  _levels_in[1]->range(0.0, 1.0);
  _levels_in[1]->label(new ui::label::Widget);
  _gamma = new ui::num::Float_Slider;
  _gamma->default_value(Color().levels.gamma);
  _gamma->range(0.0, 4.0);
  _gamma->label(new ui::label::Widget("Gamma:"));
  _levels_out[0] = new ui::num::Float_Slider;
  _levels_out[0]->default_value(Color().levels.out[0]);
  _levels_out[0]->range(0.0, 1.0);
  _levels_out[0]->label(new ui::label::Widget("Output:"));
  _levels_out[1] = new ui::num::Float_Slider;
  _levels_out[1]->default_value(Color().levels.out[1]);
  _levels_out[1]->range(0.0, 1.0);
  _levels_out[1]->label(new ui::label::Widget);
  _soft_clip = new ui::num::Float_Slider;
  _soft_clip->default_value(Color().soft_clip);
  _soft_clip->range(0.0, 1.0);
  _soft_clip->label(new ui::label::Widget("Soft Clip:"));
  
  // Layout.
  
  ui::layout::group(List<ui::layout::Item *>() << _brightness << _contrast <<
    _levels_in[0] << _levels_in[1] << _gamma << _levels_out[0] <<
    _levels_out[1] << _soft_clip);
  
  ui::layout::Vertical * layout = new ui::layout::Vertical(this);
  
  layout->add(lut_group);
  ui::layout::Vertical * group_layout = new ui::layout::Vertical(lut_group);
  ui::layout::Horizontal * h_layout = new ui::layout::Horizontal(group_layout);
  h_layout->margin(0);
  h_layout->add(_lut);
  
  layout->add(color_group);
  color_group->layout()->add(_brightness);
  color_group->layout()->add(_contrast);
  
  layout->add(levels_group);
  levels_group->layout()->add(_levels_in[0]);
  levels_group->layout()->add(_levels_in[1]);
  levels_group->layout()->add(_gamma);
  levels_group->layout()->add(_levels_out[0]);
  levels_group->layout()->add(_levels_out[1]);
  levels_group->layout()->add(_soft_clip);
  
  layout->add_stretch();
  
  // Initialize.

  widget_update();
  
  // Callbacks.
  
  _lut->sig.set(this, lut_callback);
  
  _brightness->sig.set(this, brightness_callback);
  _contrast->sig.set(this, contrast_callback);

  _levels_in[0]->sig.set(this, levels_in0_callback);
  _levels_in[1]->sig.set(this, levels_in1_callback);
  _gamma->sig.set(this, gamma_callback);
  _levels_out[0]->sig.set(this, levels_out0_callback);
  _levels_out[1]->sig.set(this, levels_out1_callback);
  _soft_clip->sig.set(this, soft_clip_callback);
}

void Widget::set(const Color & in)
{
  _value = in;
  widget_update();
}

void Widget::lut_callback(const File & in)
{
  DJV_DBG_OBJ("widget::Widget::lut_callback");
  
  _value.lut_file = in;
  _value.lut_load();
  sig.emit(_value);
}

void Widget::brightness_callback(double in)
{
  _value.color.brightness = in;
  sig.emit(_value);
}

void Widget::contrast_callback(double in)
{
  _value.color.contrast = in;
  sig.emit(_value);
}

void Widget::levels_in0_callback(double in)
{
  _value.levels.in[0] = in;
  sig.emit(_value);
}

void Widget::levels_in1_callback(double in)
{
  _value.levels.in[1] = in;
  sig.emit(_value);
}

void Widget::gamma_callback(double in)
{
  _value.levels.gamma = in;
  sig.emit(_value);
}

void Widget::levels_out0_callback(double in)
{
  _value.levels.out[0] = in;
  sig.emit(_value);
}

void Widget::levels_out1_callback(double in)
{
  _value.levels.out[1] = in;
  sig.emit(_value);
}

void Widget::soft_clip_callback(double in)
{
  _value.soft_clip = in;
  sig.emit(_value);
}

void Widget::widget_update()
{
  DJV_DBG_OBJ("widget::Widget::widget_update");
  
  callbacks(false);
  
  _lut->set(_value.lut_file);
  
  _brightness->set(_value.color.brightness);
  _contrast->set(_value.color.contrast);

  _levels_in[0]->set(_value.levels.in[0]);
  _levels_in[1]->set(_value.levels.in[1]);
  _gamma->set(_value.levels.gamma);
  _levels_out[0]->set(_value.levels.out[0]);
  _levels_out[1]->set(_value.levels.out[1]);
  _soft_clip->set(_value.soft_clip);
  
  callbacks(true);
}

}
}

