/*------------------------------------------------------------------------------
 lib/djv_base/plugin.h
 
 Plugins
 -------
 
 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_PLUGIN_H
#define DJV_PLUGIN_H

#include <djv_base/base.h>

namespace djv {
namespace plugin
{

//------------------------------------------------------------------------------
// Plugin Base
  
class Plugin
{
public:

  Plugin() throw (Error) {}
  virtual ~Plugin() = 0;
  
  virtual Str name() const = 0;
  
  virtual Plugin * copy() const = 0;
};

// Plugin entry points.

#if defined(WIN32)
#define DJV_PLUGIN_EXPORT __declspec(dllexport)
#else
#define DJV_PLUGIN_EXPORT
#endif

typedef Plugin * (Plugin_Entry)();

//------------------------------------------------------------------------------
// Plugin Factory Base

class Factory
{
public:

  Factory();
  virtual ~Factory();
  
  void set(const List<Str> & search_path_list = List<Str>());
  void add(Plugin *);
  void del();
  const List<Plugin *> & list() const { return _list; }
  List<Str> name_list() const;

protected:
  
  virtual Str plugin_prefix() const { return "djv"; }
  virtual Str plugin_entry() const = 0;

private:

  class Handle;
  
  List<Plugin *> _list;
  List<Handle *> _handle_list;
};

}
}

#endif
