/*------------------------------------------------------------------------------
 lib/djv_base/str.h
 
 Strings
 -------

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_STR_H
#define DJV_STR_H

#include <djv_base/list.h>

namespace djv {
namespace str
{

//------------------------------------------------------------------------------
// Constants

static const int cstr_size = 256;

//------------------------------------------------------------------------------
// Formatting

class Format
{
public:

  Format(const Str & in) : _value(in) {}

  Format arg(const Str &, int width = 0, char fill = ' ') const;
  Format arg(const List<Str> &, const Str & separator = " ", int width = 0,
    char fill = ' ') const;
  Format arg(int, int width = 0, char fill = ' ') const;
  Format arg(uint, int width = 0, char fill = ' ') const;
  Format arg(double, int width = 0, int precision = 0, char fill = ' ') const;
  
  operator Str () const { return _value; }
  
private:

  Str _value;
};

//------------------------------------------------------------------------------
// Functions

inline Str lower(const Str &);
inline Str upper(const Str &);

inline bool is_digit(char);

inline Str join(const List<Str> &, const Str & separator = Str());
List<Str> split(const Str &, const Str & separator, bool keep_empty = false);

//#if defined(WIN32)
//#define SNPRINTF ::sprintf_s
//#else
#define SNPRINTF ::snprintf
//#endif

List<Str> format(const Str &, int width);

//------------------------------------------------------------------------------
// Comparison

bool compare(const Str &, const Str &, bool case_insensitive = true);

//------------------------------------------------------------------------------
// Conversion

int ascii(const Str &, char *, int max_len = 0, bool terminate = true);

inline int str_to_int(const char *, int max_len = cstr_size);
inline int str_to_int(const Str &);
double str_to_float(const Str &);

int int_to_str(int, char *, int max_len = cstr_size);
Str int_to_str(int);
Str float_to_str(double, int precision = 0);

inline bool serialize(List<Str> *, Str *);
inline bool serialize(List<Str> *, bool *);
inline bool serialize(List<Str> *, int *);
inline bool serialize(List<Str> *, double *);
template <typename T>
inline bool serialize(List<Str> *, T *, const List<Str> & label);
inline bool serialize(List<Str> *, List<Str> *);
inline bool serialize(List<Str> *, List<int> *);

inline List<Str> serialize(const Str &);
inline List<Str> serialize(bool);
inline List<Str> serialize(int);
inline List<Str> serialize(double);
template <typename T>
inline List<Str> serialize(const T &, const List<Str> & label);
inline List<Str> serialize(const List<Str> &);
inline List<Str> serialize(const List<int> &);

//------------------------------------------------------------------------------
// Labels

const List<Str> & label_bool();

}
}

#include <djv_base/str_.h>

#endif

