/*------------------------------------------------------------------------------
 lib/djv_base/sys.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_base/sys.cxx"

#include "sys.h"
#include <djv_base/file.h>
#include <djv_base/app.h>
#if defined(WIN32)
#include <windows.h>
#else
#include <sys/ioctl.h>
#include <sys/termios.h>
#include <unistd.h>
#endif
#include <stdlib.h>

namespace djv {
namespace sys
{

Str env(const Str & in)
{
  const char * p = ::getenv(in.c_str());
  return Str(p ? p : "");
}

List<Str> search_path()
{
  DJV_DBG_OBJ2(_DBG, "search_path");
  
  List<Str> out;

  // Installation path.

#if defined(DJV_SEARCH_PATH)
  
  DJV_DBG_PRN("DJV_SEARCH_PATH = " << DJV_SEARCH_PATH);
  out += str::split(DJV_SEARCH_PATH, file::list_separator_list);
  
#endif

  // DJV_PATH environment variable.

  Str path = sys::env("DJV_PATH");
  if (path.size())
  {
    DJV_DBG_PRN("DJV_PATH = " << path);
    out += str::split(path, file::list_separator_list);
  }

  // System environment variables.

  path = sys::env("LD_LIBRARY_PATH");
  if (path.size())
  {
    DJV_DBG_PRN("LD_LIBRARY_PATH = " << path);
    out += str::split(path, file::list_separator_list);
  }
  
  // Application path.
  
  const Str & cmdln_name = app::global()->cmdln_name();
  path = File(cmdln_name).path();
  if (path.size())
  {
    path = file::path_absolute(path);
    DJV_DBG_PRN("app path = " << path);
    out += path;
  }
  else
  {
    const List<Str> tmp = List<Str>() <<
      str::split(sys::env("PATH"), file::list_separator_list) <<
      ".";
    for (uint i = 0; i < tmp.size(); ++i)
    {
      path = file::path_absolute(tmp[i]);
        
      // XXX
        
      const Str name = path + file::path_separator + cmdln_name;
      if (File(name).stat() || File(name + ".exe").stat())
      {
        DJV_DBG_PRN("app path = " << path);
        out += path;
        break;
      }
    }
  }
  
  // Current directory.

  //out += ".";

  DJV_DBG_PRN("out = " << out);
  return out;
}

int terminal_width()
{
  int out = 80;

#if defined(WIN32)

  HANDLE h = ::GetStdHandle(STD_OUTPUT_HANDLE);
  if (h != INVALID_HANDLE_VALUE)
  {
    CONSOLE_SCREEN_BUFFER_INFO info;
    if (::GetConsoleScreenBufferInfo(h, &info))
      out = info.dwSize.X;
  }

#else

  struct winsize ws;
  ws.ws_col = 0;
  if (ioctl(STDOUT_FILENO, TIOCGWINSZ, &ws) == -1 &&
    ioctl(STDERR_FILENO, TIOCGWINSZ, &ws) == -1 &&
    ioctl(STDIN_FILENO, TIOCGWINSZ, &ws) == -1)
    ws.ws_col = 80;
  out = ws.ws_col;

#endif

  return out;
}

}
}
