/*------------------------------------------------------------------------------
 lib/djv_image/io_lut.h
 
 Lookup Table (LUT) File I/O
 ---------------------------

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_IMAGE_IO_LUT_H
#define DJV_IMAGE_IO_LUT_H

#include <djv_image/io.h>

namespace djv {
namespace image {
namespace io {
namespace lut
{

//------------------------------------------------------------------------------
// Plugin Information

static const Str name = "LUT";

static const List<Str> extension_list = List<Str>() << ".lut" << ".1dl";

//------------------------------------------------------------------------------
// Plugin Options

enum SPEC { SPEC_AUTO, SPEC_INFERNO, SPEC_KODAK };
enum TYPE { TYPE_AUTO, TYPE_U8, TYPE_U10, TYPE_U16 };

//------------------------------------------------------------------------------
// Load Plugin

class Load : public io::Load
{
public:
  
  Plugin * copy() const;
  
  Str name() const { return lut::name; }
  const List<Str> & extension_list() const { return lut::extension_list; }
  Str help() const;
  
  struct Options
  {
    Options() : spec(SPEC_AUTO), type(TYPE_AUTO) {}
    SPEC spec;
    TYPE type;
  };
  enum OPTION { OPTION_SPEC, OPTION_TYPE };
  bool option(const Str &, List<Str> *);
  List<Str> option(const Str &) const;
  const List<Str> & option_list() const;
  
  void open(const File &, Info *, Tag *, Time *) throw (Error);
  void open(const File & in, List<Info> * info, Tag * tag, Time * time)
    throw (Error)
  { io::Load::open(in, info, tag, time); }
  const Image * get(int frame, int layer, int proxy) throw (Error);

private:

  void _open(const File &, Info *) throw (Error);

  Options _options;
  File _file;
  djv::Io _io;
  SPEC _spec;
  Image _image;
};

//------------------------------------------------------------------------------
// Save Plugin

class Save : public io::Save
{
public:
  
  Plugin * copy() const;
  
  Str name() const { return lut::name; }
  const List<Str> & extension_list() const { return lut::extension_list; }
  Str help() const;
  
  struct Options
  {
    Options() : spec(SPEC_AUTO) {}
    SPEC spec;
  };
  enum OPTION { OPTION_SPEC };
  bool option(const Str &, List<Str> *);
  List<Str> option(const Str &) const;
  const List<Str> & option_list() const;
  
  void open(const File &, const Info &, const Tag &, const Time &)
    throw (Error);
  void set(const Image &, int frame) throw (Error);

private:

  void _open(const File &) throw (Error);
  
  Options _options;
  File _file;
  djv::Io _io;
  SPEC _spec;
  Info _info;
  Image _image;
};

//------------------------------------------------------------------------------
// Labels

const List<Str> & label_spec();
const List<Str> & label_type();
const List<Str> & label_load_option();
const List<Str> & label_save_option();

}
}
}
}

#endif

