/*------------------------------------------------------------------------------
 lib/djv_image/io_tga.h

 TARGA Image I/O
 ---------------
 
 References:
 * James D. Murray, William vanRyper, "Encyclopedia of Graphics File Formats,
   Second Edition"

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_IMAGE_IO_TGA_H
#define DJV_IMAGE_IO_TGA_H

#include <djv_image/io.h>

namespace djv {
namespace image {
namespace io {
namespace tga
{

//------------------------------------------------------------------------------
// Plugin Information

static const Str name = "TGA";

static const List<Str> extension_list = List<Str>() << ".tga";

//------------------------------------------------------------------------------
// Plugin Options

enum COMPRESS { COMPRESS_NONE, COMPRESS_RLE };

//------------------------------------------------------------------------------
// Load Plugin

class Load : public io::Load
{
public:

  Load();

  Plugin * copy() const { return new Load; }
  
  Str name() const { return tga::name; }
  const List<Str> & extension_list() const { return tga::extension_list; }
  
  void open(const File &, Info *, Tag *, Time *) throw (Error);
  void open(const File & in, List<Info> * info, Tag * tag, Time * time)
    throw (Error)
  { io::Load::open(in, info, tag, time); }
  const Image * get(int frame, int layer, int proxy) throw (Error);

private:

  void _open(const File &, Info *) throw (Error);

  File _file;
  djv::Io _io;
  bool _compress;
  Image _image;
  Data _tmp;
};

//------------------------------------------------------------------------------
// Save Plugin

class Save : public io::Save
{
public:

  Save();
  
  Plugin * copy() const;
  
  Str name() const { return tga::name; }
  const List<Str> & extension_list() const { return tga::extension_list; }
  Str help() const;
  
  struct Options
  {
    Options() : compress(COMPRESS_NONE) {}
    COMPRESS compress;
  };
  enum OPTION { OPTION_COMPRESS };
  bool option(const Str &, List<Str> *);
  List<Str> option(const Str &) const;
  const List<Str> & option_list() const;
  
  void open(const File &, const Info &, const Tag &, const Time &)
    throw (Error);
  void set(const Image &, int frame) throw (Error);

private:
  
  void _open(const File &) throw (Error);

  Options _options;
  File _file;
  djv::Io _io;
  Info _info;
  Image _image;
};

//------------------------------------------------------------------------------
// Functions

void rle_unpack(djv::Io *, void *, int size, int channels) throw (Error);
size_t rle_pack(const void * in, void * out, int size, int channels);

//------------------------------------------------------------------------------
// Labels

const List<Str> & label_compress();
const List<Str> & label_save_option();

}
}
}
}

#endif
