/*------------------------------------------------------------------------------
 lib/djv_image/pixel_.h

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include <djv_base/math.h>

namespace djv {
namespace image {
namespace pixel
{

inline FORMAT format(PIXEL in)
{
  static const FORMAT data [] = {
    L, L, L, L,
    LA, LA, LA, LA,
    RGB, RGB, RGB, RGB, RGB,
    RGBA, RGBA, RGBA, RGBA };
  return data[in];
}

inline TYPE type(PIXEL in)
{
  static const TYPE data [] = {
    U8, U16, F16, F32,
    U8, U16, F16, F32,
    U8, U10, U16, F16, F32,
    U8, U16, F16, F32 };
  return data[in];
}

inline PIXEL pixel(FORMAT format, TYPE type)
{
  PIXEL out = PIXEL(0);
  switch (format)
  {
    case L:
      switch (type)
      {
        case U8: out = L_U8; break;
        case U10:
        case U16: out = L_U16; break;
        case F16: out = L_F16; break;
        case F32: out = L_F32; break;
      }
      break;
    case LA:
      switch (type)
      {
        case U8: out = LA_U8; break;
        case U10:
        case U16: out = LA_U16; break;
        case F16: out = LA_F16; break;
        case F32: out = LA_F32; break;
      }
      break;
    case RGB:
      switch (type)
      {
        case U8: out = RGB_U8; break;
        case U10: out = RGB_U10; break;
        case U16: out = RGB_U16; break;
        case F16: out = RGB_F16; break;
        case F32: out = RGB_F32; break;
      }
      break;
    case RGBA:
      switch (type)
      {
        case U8: out = RGBA_U8; break;
        case U10:
        case U16: out = RGBA_U16; break;
        case F16: out = RGBA_F16; break;
        case F32: out = RGBA_F32; break;
      }
      break;
  }
  return out;
}

inline bool pixel(FORMAT format, TYPE type, PIXEL * out)
{
  switch (format)
  {
    case L:
      switch (type)
      {
        case U8: *out = L_U8; break;
        case U16: *out = L_U16; break;
        case F16: *out = L_F16; break;
        case F32: *out = L_F32; break;
        default: return false;
      }
      break;
    case LA:
      switch (type)
      {
        case U8: *out = LA_U8; break;
        case U16: *out = LA_U16; break;
        case F16: *out = LA_F16; break;
        case F32: *out = LA_F32; break;
        default: return false;
      }
      break;
    case RGB:
      switch (type)
      {
        case U8: *out = RGB_U8; break;
        case U10: *out = RGB_U10; break;
        case U16: *out = RGB_U16; break;
        case F16: *out = RGB_F16; break;
        case F32: *out = RGB_F32; break;
        default: return false;
      }
      break;
    case RGBA:
      switch (type)
      {
        case U8: *out = RGBA_U8; break;
        case U16: *out = RGBA_U16; break;
        case F16: *out = RGBA_F16; break;
        case F32: *out = RGBA_F32; break;
        default: return false;
      }
      break;
    default: return false;
  }
  return true;
}

inline bool pixel(int channels, int bit_depth, PIXEL * out)
{
  switch (channels)
  {
    case 1:
      switch (bit_depth)
      {
        case 8: *out = L_U8; break;
        case 16: *out = L_U16; break;
        case 32: *out = L_F32; break;
        default: return false;
      }
      break;
    case 2:
      switch (bit_depth)
      {
        case 8: *out = LA_U8; break;
        case 16: *out = LA_U16; break;
        case 32: *out = LA_F32; break;
        default: return false;
      }
      break;
    case 3:
      switch (bit_depth)
      {
        case 8: *out = RGB_U8; break;
        case 10: *out = RGB_U10; break;
        case 16: *out = RGB_U16; break;
        case 32: *out = RGB_F32; break;
        default: return false;
      }
      break;
    case 4:
      switch (bit_depth)
      {
        case 8: *out = RGBA_U8; break;
        case 16: *out = RGBA_U16; break;
        case 32: *out = RGBA_F32; break;
        default: return false;
      }
      break;
    default: return false;
  }
  return true;
}

inline int pixel(int channels, int bit_depth)
{
  PIXEL tmp = PIXEL(0);
  if (! pixel(channels, bit_depth, &tmp)) return -1;
  return tmp;
}

inline int channels(FORMAT in)
{
  static const int data [] = { 1, 2, 3, 4 };
  return data[in];
}

inline int channels(PIXEL in)
{
  static const int data [] = {
    1, 1, 1, 1,
    2, 2, 2, 2,
    3, 3, 3, 3, 3,
    4, 4, 4, 4 };
  return data[in];
}

inline int channel_bytes(PIXEL in)
{
  static const int data [] = {
    1, 2, 2, 4,
    1, 2, 2, 4,
    1, 0, 2, 2, 4,
    1, 2, 2, 4 };
  return data[in];
}

inline int bytes(PIXEL in)
{
  static const int data [] = {
    1, 2, 2, 4,
    2, 4, 4, 8,
    3, 4, 6, 6, 12,
    4, 8, 8, 16 };
  return data[in];
}

inline int bit_depth(PIXEL in)
{
  static const int data [] = {
    8, 16, 16, 32,
    8, 16, 16, 32,
    8, 10, 16, 16, 32,
    8, 16, 16, 32 };
  return data[in];
}

inline int range(PIXEL in)
{
  static const int data [] = {
    u8_range, u16_range, 1, 1,
    u8_range, u16_range, 1, 1,
    u8_range, u10_range, u16_range, 1, 1,
    u8_range, u16_range, 1, 1 };
  return data[in];
}

inline int max(PIXEL in)
{
  static const int data [] = {
    u8_max, u16_max, 1, 1,
    u8_max, u16_max, 1, 1,
    u8_max, u10_max, u16_max, 1, 1,
    u8_max, u16_max, 1, 1 };
  return data[in];
}

// XXX Use a LUT because shifting doesn't preserve max values.

#define  _PIXEL_U8_RANGE  u8_range
#define _PIXEL_U10_RANGE u10_range
#define _PIXEL_U16_RANGE u16_range

#define _PIXEL_LUT(IN, OUT, RANGE, MAX) \
  \
  static OUT##_T lut [_PIXEL_##IN##_RANGE]; \
  static bool init = true; \
  if (init) \
  { \
    for (int i = 0; i < RANGE; ++i) \
      lut[i] = OUT##_T(i / float(RANGE - 1) * MAX); \
    init = false; \
  } \
  return lut[in];

inline U10_T u8_to_u10(U8_T in)
{
  //return in << 2;
  _PIXEL_LUT(U8, U10, u8_range, u10_max)
}

inline U16_T u8_to_u16(U8_T in)
{
  //return in << 8;
  _PIXEL_LUT(U8, U16, u8_range, u16_max)
}

inline F16_T u8_to_f16(U8_T in)
{
  //return in / F16(u8_max);
  _PIXEL_LUT(U8, F16, u8_range, 1.0)
}

inline F32_T u8_to_f32(U8_T in)
{
  //return in / F32(u8_max);
  _PIXEL_LUT(U8, F32, u8_range, 1.0)
}

inline U8_T u10_to_u8(U10_T in) { return in >> 2; }

inline U16_T u10_to_u16(U10_T in)
{
  //return in << 6;
  _PIXEL_LUT(U10, U16, u10_range, u16_max)
}

inline F16_T u10_to_f16(U10_T in)
{
  //return in / F16(u10_max);
  _PIXEL_LUT(U10, F16, u10_range, 1.0)
}

inline F32_T u10_to_f32(U10_T in)
{
  //return in / F32(u10_max);
  _PIXEL_LUT(U10, F32, u10_range, 1.0)
}

inline U8_T u16_to_u8(U16_T in) { return in >> 8; }
inline U10_T u16_to_u10(U16_T in) { return in >> 6; }

inline F16_T u16_to_f16(U16_T in)
{
  //return in / F16(u16_max);
  _PIXEL_LUT(U16, F16, u16_range, 1.0);
}

inline F32_T u16_to_f32(U16_T in)
{
  //return in / F32(u16_max);
  _PIXEL_LUT(U16, F32, u16_range, 1.0)
}

inline U8_T f16_to_u8(F16_T in)
{ return math::clamp(int(in * u8_max + 0.5), 0, u8_max); }
inline U10_T f16_to_u10(F16_T in)
{ return math::clamp(int(in * u10_max + 0.5), 0, u10_max); }
inline U16_T f16_to_u16(F16_T in)
{ return math::clamp(int(in * u16_max + 0.5), 0, u16_max); }
inline F32_T f16_to_f32(F16_T in)
{ return in; }

inline U8_T f32_to_u8(F32_T in)
{
  //DJV_DBG_FNC("djv::image::pixel::f32_to_u8");
  //DJV_DBG_PRN("in = " << in);
  
  return math::clamp(int(in * u8_max + 0.5), 0, u8_max);

  /*// XXX Ill floating point conversion. Assumes IEEE floats: sign (1 bit)
  // + exponent (8 bits) + mantissa (23 bits).
  //
  // References:
  // * http://www.stereopsis.com/FPU.html

  // Transform to get a constant exponent of 127 for 0.0 to 1.0 range.

  const F32_T tmp = in + 1.0;
  const uint32_t bits = *(const uint32_t *)&tmp;
  //DJV_DBG_PRN(_bits(tmp));
  //DJV_DBG_PRN(_bits(bits));

  // Bail on negative numbers.

  if (bits >> 31) return 0;

  // Exponent:
  // * < 127 - Input <= 0.0.
  // * 127 - Input between 0.0 and 1.0. Mantissa becomes 23 bit integer.
  // * > 127 - Input >= 1.0.

  const uint8_t exp = (bits >> 23) & 0xff;
  //DJV_DBG_PRN("exp = " << exp);
  if (127 == exp) return (bits >> 15) & 0xff;
  else if (exp > 127) return u8_max;
  else return 0;*/
}

inline U10_T f32_to_u10(F32_T in)
{ return math::clamp(int(in * u10_max + 0.5), 0, u10_max); }
inline U16_T f32_to_u16(F32_T in)
{ return math::clamp(int(in * u16_max + 0.5), 0, u16_max); }
inline F16_T f32_to_f16(F32_T in)
{ return in; }

}
}
}

