/*------------------------------------------------------------------------------
 lib/djv_image/profile.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include "profile.h"

namespace djv {
namespace image {
namespace profile
{

bool operator == (const Exposure & a, const Exposure & b)
{
  return a.value == b.value && a.defog == b.defog &&
    a.knee_low == b.knee_low && a.knee_high == b.knee_high &&
    a.gamma == b.gamma;
}

bool operator != (const Exposure & a, Exposure & b) { return ! (a == b); }

bool operator == (const Profile & a, const Profile & b)
{
  return a.type == b.type && a.exposure == b.exposure;
}

bool operator != (const Profile & a, const Profile & b) { return ! (a == b); }

bool serialize(List<Str> * in, Exposure * out)
{
  if (! str::serialize(in, &out->value)) return false;
  if (! str::serialize(in, &out->defog)) return false;
  if (! str::serialize(in, &out->knee_low)) return false;
  if (! str::serialize(in, &out->knee_high)) return false;
  if (! str::serialize(in, &out->gamma)) return false;
  return true;
}

List<Str> serialize(const Exposure & in)
{
  return List<Str>() << str::serialize(in.value) << str::serialize(in.defog) <<
    str::serialize(in.knee_low) << str::serialize(in.knee_high) <<
    str::serialize(in.gamma);
}

const List<Str> & label_profile()
{
  static const List<Str> data = List<Str>() << "Raw" << "LUT" << "Exposure";
  return data;
}

Dbg & operator << (Dbg & dbg, PROFILE & in)
{ return dbg << label_profile()[in]; }

}
}
}

