/*------------------------------------------------------------------------------
 lib/djv_image/time.h
 
 Image Time
 ----------
 
 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_IMAGE_TIME_H
#define DJV_IMAGE_TIME_H

#include <djv_image/base.h>

namespace djv {
namespace image {
namespace time
{

//------------------------------------------------------------------------------
// Speed

enum FPS { FPS_1, FPS_3, FPS_6, FPS_12, FPS_15, FPS_16, FPS_18, FPS_23_98,
  FPS_24, FPS_25, FPS_29_97, FPS_30, FPS_50, FPS_59_94, FPS_60, FPS_120 };

struct Speed
{
  Speed() : scale(0), duration(1) {}
  Speed(int scale, int duration = 1) : scale(scale), duration(duration) {}
  Speed(FPS);
  
  bool valid() const { return scale != 0 && duration != 0; }
  
  int scale, duration;
};

//------------------------------------------------------------------------------
// Time

struct Time
{
  Time();
  Time(int start, int end);
  Time(int start, int end, const Speed &);

  int start, end;
  Speed speed;
};

//------------------------------------------------------------------------------
// Timecode

void timecode_to_time(uint32_t, int * hour, int * minute, int * second,
  int * frame);
uint32_t time_to_timecode(int hour, int minute, int second, int frame);

int timecode_to_frame(uint32_t timecode, const time::Speed &);
uint32_t frame_to_timecode(int frame, const time::Speed &);

Str timecode_to_str(uint32_t);
uint32_t str_to_timecode(const Str &);

//------------------------------------------------------------------------------
// Keycode

Str keycode_to_str(int id, int type, int prefix, int count, int offset);
bool str_to_keycode(const Str &, int * id, int * type, int * prefix,
  int * count, int * offset);

//------------------------------------------------------------------------------
// Comparison

bool operator == (const Speed &, const Speed &);
bool operator == (const Time &, const Time &);
bool operator != (const Speed &, const Speed &);
bool operator != (const Time &, const Time &);

//------------------------------------------------------------------------------
// Conversion

double speed_to_float(const Speed &);
Speed float_to_speed(double);

bool serialize(List<Str> *, Speed *);
bool serialize(List<Str> *, Time *);
List<Str> serialize(const Speed &);
List<Str> serialize(const Time &);

//------------------------------------------------------------------------------
// Defaults

void speed(const Speed &);
const Speed & speed();

//------------------------------------------------------------------------------
// Labels

const List<Str> & label_fps();

//------------------------------------------------------------------------------
// Debugging

Dbg & operator << (Dbg &, const Speed &);
Dbg & operator << (Dbg &, const Time &);

}
}
}

#endif

