/*------------------------------------------------------------------------------
 lib/djv_ui/about_dialog.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include "about_dialog.h"
#include <djv_ui/style.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/group.h>
#include <djv_ui/label.h>
#include <djv_ui/button.h>
#include <djv_ui/text.h>
#include <djv_ui/app.h>
#include <FL/Fl.H>
#include <FL/Fl_Pixmap.H>
#include <FL/fl_draw.H>
#include "durant.xpm"

namespace djv {
namespace ui {
namespace about_dialog
{

class _Widget : public text::Multiline_Display
{
public:

  _Widget(const Str & in) : text::Multiline_Display(in) {}
};

Dialog::Dialog() : dialog::Base("About")
{
  // Widgets.
  
  _image = std::auto_ptr<Fl_Image>(new Fl_Pixmap((const char **)durant_xpm));
  ui::label::Image * image_label = new ui::label::Image(_image.get());

  _Widget * widget = new _Widget(
    str::join(str::format(app::global()->about(), 80), "\n"));
  
  button::Push * close_button = new button::Push("Clos&e");
    
  // Layout.
  
  layout::Vertical * layout = new layout::Vertical(this);
  
  layout->add(image_label);
  layout->add(widget);
  layout->stretch(widget);
  
  layout::Horizontal * h_layout = new layout::Horizontal(layout);
  h_layout->margin(0);
  h_layout->add_stretch();
  h_layout->add(close_button);
  
  // Initialize.
  
  size(size_hint());
  close_button->take_focus();

  // Callbacks.

  close_button->sig.set(this, close_callback);
}

void Dialog::close_callback(bool)
{
  hide();
}

namespace { std::auto_ptr<Dialog> _dialog; }

Dialog * global()
{
  if (! _dialog.get()) _dialog = std::auto_ptr<Dialog>(new Dialog);
  return _dialog.get();
}

void del()
{
  if (_dialog.get()) _dialog->widget_del();
  _dialog.release();
}

}
}
}

