/*------------------------------------------------------------------------------
 lib/djv_ui/button.h

 Button Widgets
 --------------

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_BUTTON_H
#define DJV_UI_BUTTON_H

#include <djv_ui/widget.h>

namespace djv {
namespace ui {
namespace button
{

//------------------------------------------------------------------------------
// Push Button

class Push : public widget::Widget
{
public:

  Push(const Str & in);
  virtual ~Push() {}
  
  // Value.

  Signal<bool> sig;
  
  // Label.

  void label(const Str &);
  const Str & label() const { return _label; }
  
  // Widget.
  
  virtual void draw();
  
  // Layout.

  virtual void dirty();
  virtual void geom(const Box2i &);
  virtual const Box2i & geom() const { return widget::Widget::geom(); }
  
  // Debugging.

  virtual Str dbg_name() const { return "djv::ui::button::Push"; }

private:

  class Widget;
  
  DJV_FL_WIDGET_CALLBACK(Push, widget_callback);
  
  Str _label;
  mem::Buffer<char> _label_ascii;
  Widget * _widget;
};

//------------------------------------------------------------------------------
// Check Button

class Check : public widget::Widget
{
public:

  Check(const Str & = Str());
  virtual ~Check() {}
  
  // Value.

  void set(bool);
  bool get() const { return _value; }
  Signal<bool> sig;
  
  // Label.

  void label(const Str &);
  const Str & label() const { return _label; }
  
  // Widget.
  
  virtual void draw();
  virtual void widget_del();
  
  // Layout.

  virtual void dirty();
  virtual void geom(const Box2i &);
  virtual const Box2i & geom() const { return widget::Widget::geom(); }
  
  // Debugging.

  virtual Str dbg_name() const { return "djv::ui::button::Check"; }
  
private:

  class Widget;
  
  DJV_FL_WIDGET_CALLBACK(Check, widget_callback);

  bool _value;
  Str _label, _label_tmp;
  mem::Buffer<char> _label_ascii;
  Widget * _widget;
};

//------------------------------------------------------------------------------
// Radio Button

class Radio : public widget::Widget
{
public:

  Radio(const Str & = Str());
  virtual ~Radio() {}
  
  // Value.

  void set(bool);
  bool get() const { return _value; }
  Signal<bool> sig;
  
  // Label.

  void label(const Str &);
  const Str & label() const { return _label; }

  // Group.
  
  void group(const List<Radio *> &);
  const List<Radio *> & group() const { return _group; }
  
  // Widget.
  
  virtual void draw();
  virtual void widget_del();
  
  // Layout.

  virtual void dirty();
  virtual void geom(const Box2i &);
  virtual const Box2i & geom() const { return widget::Widget::geom(); }
  
  // Debugging.

  virtual Str dbg_name() const { return "djv::ui::button::Radio"; }
  
private:

  class Widget;
  
  DJV_FL_WIDGET_CALLBACK(Radio, widget_callback);
  
  void widget_update();

  bool _value;
  Str _label, _label_tmp;
  mem::Buffer<char> _label_ascii;
  List<Radio *> _group;
  Widget * _widget;
};

//------------------------------------------------------------------------------
// Tool Button

class Tool : public widget::Widget
{
public:

  Tool(const Str & icon, int margin = 2);
  virtual ~Tool() {}
  
  // Value.

  void set(bool = true);
  bool get() const { return _value; }
  Signal<bool> sig;
  
  // Type.

  enum TYPE { PUSH, TOGGLE, RADIO, REPEAT };
  void type(TYPE);
  TYPE type() const { return _type; }

  void radio_group(const List<Tool *> &);
  const List<Tool *> & radio_group() const { return _radio_group; }
  
  // Icon.

  void icon(Fl_Image *);
  
  // Shortcut.

  void shortcut(int);
  int shortcut() const { return _shortcut; }
  
  // Widget.

  virtual int handle(int);
  virtual void draw();
  virtual void widget_del();
  
  // Layout.
  
  void margin(int);
  int margin() const { return _margin; }

  virtual void dirty();
  
  // Debugging.

  virtual Str dbg_name() const { return "djv::ui::button::Tool"; }

private:

  void action();
  
  DJV_FL_CALLBACK(Tool, repeat_callback);

  bool _value;
  TYPE _type;
  List<Tool *> _radio_group;
  bool _repeat_active;
  Fl_Image * _icon;
  std::auto_ptr<Fl_Image> _icon_inactive;
  int _margin;
  int _shortcut;
  bool _pressed, _pressed_inside;
};

//------------------------------------------------------------------------------
// Functions

void radio_group(const List<Radio *> &);
void radio_group(const List<Tool *> &);

}
}
}

#endif

