/*------------------------------------------------------------------------------
 lib/djv_ui/image_io.h
 
 Image I/O Widgets
 -----------------
 
 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_IMAGE_IO_H
#define DJV_UI_IMAGE_IO_H

#include <djv_base/plugin.h>
#include <djv_image/io.h>
#include <djv_ui/widget.h>

namespace djv {
namespace ui {
namespace image_io
{

//------------------------------------------------------------------------------
// Widget Base

class Base : public widget::Widget, public plugin::Plugin
{
public:

  virtual ~Base() {}
  
  virtual void set(djv::image::io::Plugin *) = 0;
  
  Signal<djv::image::io::Plugin *> sig;
};

//------------------------------------------------------------------------------
// Load Widget

class Load : public Base {};

//------------------------------------------------------------------------------
// Save Widget

class Save : public Base {};

//------------------------------------------------------------------------------
// Load Factory

class Load_Factory : public plugin::Factory
{
public:

  Load_Factory();
  ~Load_Factory();
  
  Load * get(djv::image::io::Load *) const;
  List<Load *> list() const;
  
private:

  Str plugin_entry() const { return "djv_ui_image_load"; }
};

//------------------------------------------------------------------------------
// Save Factory

class Save_Factory : public plugin::Factory
{
public:

  Save_Factory();
  ~Save_Factory();
  
  Save * get(djv::image::io::Save *) const;
  List<Save *> list() const;
  
private:

  Str plugin_entry() const { return "djv_ui_image_save"; }
};

//------------------------------------------------------------------------------
// Global

Load_Factory * load();
Save_Factory * save();

extern Signal<djv::image::io::Plugin *> sig;

void init();
void del();

}
}
}

#endif

