/*------------------------------------------------------------------------------
 lib/djv_ui/image_io_builtin.h
 
 Built-In Image I/O Widgets
 --------------------------
 
 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_IMAGE_IO_BUILTIN_H
#define DJV_UI_IMAGE_IO_BUILTIN_H

#include <djv_image/io_cineon.h>
#include <djv_image/io_dpx.h>
#include <djv_image/io_lut.h>
#include <djv_image/io_ppm.h>
#include <djv_image/io_sgi.h>
#include <djv_image/io_tga.h>
#include <djv_ui/image_io.h>

namespace djv {
namespace ui {
namespace image_io
{

//------------------------------------------------------------------------------
// Cineon Image I/O Widgets

class Cineon_Load : public image_io::Load
{
public:

  Cineon_Load();
  
  plugin::Plugin * copy() const { return new Cineon_Load; }

  Str name() const { return djv::image::io::cineon::name; }
  
  void set(djv::image::io::Plugin *);
  
private:

  DJV_CALLBACK(Cineon_Load, color_callback, int);
  DJV_CALLBACK(Cineon_Load, black_callback, int);
  DJV_CALLBACK(Cineon_Load, white_callback, int);
  DJV_CALLBACK(Cineon_Load, gamma_callback, double);
  DJV_CALLBACK(Cineon_Load, soft_clip_callback, int);
  
  void plugin_update();
  void widget_update();
  
  djv::image::io::Plugin * _plugin;
  djv::image::io::cineon::Load::Options _options;
  choice::Widget * _color_widget;
  num::Int_Slider * _black_widget, * _white_widget;
  num::Float_Slider * _gamma_widget;
  num::Int_Slider * _soft_clip_widget;
};

class Cineon_Save : public image_io::Save
{
public:

  Cineon_Save();

  plugin::Plugin * copy() const { return new Cineon_Save; }

  Str name() const { return djv::image::io::cineon::name; }

  void set(djv::image::io::Plugin *);
  
private:

  DJV_CALLBACK(Cineon_Save, color_callback, int);
  DJV_CALLBACK(Cineon_Save, black_callback, int);
  DJV_CALLBACK(Cineon_Save, white_callback, int);
  DJV_CALLBACK(Cineon_Save, gamma_callback, double);
  
  void plugin_update();
  void widget_update();

  djv::image::io::Plugin * _plugin;
  djv::image::io::cineon::Save::Options _options;
  choice::Widget * _color_widget;
  num::Int_Slider * _black_widget, * _white_widget;
  num::Float_Slider * _gamma_widget;
};

//------------------------------------------------------------------------------
// DPX Image I/O Widgets

class Dpx_Load : public image_io::Load
{
public:

  Dpx_Load();
  
  plugin::Plugin * copy() const { return new Dpx_Load; }

  Str name() const { return djv::image::io::dpx::name; }
  
  void set(djv::image::io::Plugin *);
  
private:

  DJV_CALLBACK(Dpx_Load, color_callback, int);
  DJV_CALLBACK(Dpx_Load, black_callback, int);
  DJV_CALLBACK(Dpx_Load, white_callback, int);
  DJV_CALLBACK(Dpx_Load, gamma_callback, double);
  DJV_CALLBACK(Dpx_Load, soft_clip_callback, int);
  
  void plugin_update();
  void widget_update();
  
  djv::image::io::Plugin * _plugin;
  djv::image::io::dpx::Load::Options _options;
  choice::Widget * _color_widget;
  num::Int_Slider * _black_widget, * _white_widget;
  num::Float_Slider * _gamma_widget;
  num::Int_Slider * _soft_clip_widget;
};

class Dpx_Save : public image_io::Save
{
public:

  Dpx_Save();

  plugin::Plugin * copy() const { return new Dpx_Save; }

  Str name() const { return djv::image::io::dpx::name; }

  void set(djv::image::io::Plugin *);
  
private:

  DJV_CALLBACK(Dpx_Save, color_callback, int);
  DJV_CALLBACK(Dpx_Save, black_callback, int);
  DJV_CALLBACK(Dpx_Save, white_callback, int);
  DJV_CALLBACK(Dpx_Save, gamma_callback, double);
  DJV_CALLBACK(Dpx_Save, type_callback, int);
  DJV_CALLBACK(Dpx_Save, endian_callback, int);
  
  void plugin_update();
  void widget_update();

  djv::image::io::Plugin * _plugin;
  djv::image::io::dpx::Save::Options _options;
  choice::Widget * _color_widget;
  num::Int_Slider * _black_widget, * _white_widget;
  num::Float_Slider * _gamma_widget;
  choice::Widget * _type_widget;
  choice::Widget * _endian_widget;
};

//------------------------------------------------------------------------------
// LUT Image I/O Widgets

class Lut_Load : public image_io::Load
{
public:

  Lut_Load();

  plugin::Plugin * copy() const { return new Lut_Load; }

  Str name() const { return djv::image::io::lut::name; }
  
  void set(djv::image::io::Plugin *);

private:

  DJV_CALLBACK(Lut_Load, spec_callback, int);
  DJV_CALLBACK(Lut_Load, type_callback, int);
  
  void plugin_update();
  void widget_update();

  djv::image::io::Plugin * _plugin;
  djv::image::io::lut::Load::Options _options;
  choice::Widget * _spec_widget;
  choice::Widget * _type_widget;
};

class Lut_Save : public image_io::Save
{
public:

  Lut_Save();

  plugin::Plugin * copy() const { return new Lut_Save; }

  Str name() const { return djv::image::io::lut::name; }
  
  void set(djv::image::io::Plugin *);

private:

  DJV_CALLBACK(Lut_Save, spec_callback, int);
  
  void plugin_update();
  void widget_update();

  djv::image::io::Plugin * _plugin;
  djv::image::io::lut::Save::Options _options;
  choice::Widget * _spec_widget;
};

//------------------------------------------------------------------------------
// PPM Image I/O Widgets

class Ppm_Save : public image_io::Save
{
public:

  Ppm_Save();

  plugin::Plugin * copy() const { return new Ppm_Save; }

  Str name() const { return djv::image::io::ppm::name; }
  
  void set(djv::image::io::Plugin *);

private:

  DJV_CALLBACK(Ppm_Save, type_callback, int);
  DJV_CALLBACK(Ppm_Save, mode_callback, int);
  
  void plugin_update();
  void widget_update();

  djv::image::io::Plugin * _plugin;
  djv::image::io::ppm::Save::Options _options;
  choice::Widget * _type_widget;
  choice::Widget * _mode_widget;
};

//------------------------------------------------------------------------------
// SGI Image I/O Widgets

class Sgi_Save : public image_io::Save
{
public:

  Sgi_Save();

  plugin::Plugin * copy() const { return new Sgi_Save; }

  Str name() const { return djv::image::io::_sgi::name; }
  
  void set(djv::image::io::Plugin *);
  
private:

  DJV_CALLBACK(Sgi_Save, compress_callback, int);
  
  void plugin_update();
  void widget_update();

  djv::image::io::Plugin * _plugin;
  djv::image::io::_sgi::Save::Options _options;
  choice::Widget * _compress_widget;
};

//------------------------------------------------------------------------------
// TGA Image I/O Widgets

class Tga_Save : public image_io::Save
{
public:

  Tga_Save();
  ~Tga_Save();

  plugin::Plugin * copy() const { return new Tga_Save; }

  Str name() const { return djv::image::io::tga::name; }
  
  void set(djv::image::io::Plugin *);
  
private:

  DJV_CALLBACK(Tga_Save, compress_callback, int);
  
  void plugin_update();
  void widget_update();

  djv::image::io::Plugin * _plugin;
  djv::image::io::tga::Save::Options _options;
  choice::Widget * _compress_widget;
};

}
}
}

#endif

