/*------------------------------------------------------------------------------
 lib/djv_ui/num.h
 
 Numeric Value Widgets
 ---------------------

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_NUM_H
#define DJV_UI_NUM_H

#include <djv_ui/widget.h>

namespace djv {
namespace ui {
namespace num
{

//------------------------------------------------------------------------------
// Widget Base

template<class T>
class Base
{
public:

  inline Base();
  inline virtual ~Base() = 0;

  // Value.

  virtual void set(T);
  const T & get() const { return _value; }
  Signal<T> sig;
  
  virtual void default_valid(bool);
  virtual void default_value(T);
  bool default_valid() const { return _default_valid; }
  const T & default_value() const { return _default_value; }
  
  // Range.

  virtual void range(T min, T max);
  const T & min() const { return _min; }
  const T & max() const { return _max; }
  
  // Increment.

  virtual void inc(T inc_small, T inc_large);
  const T & inc_small() const { return _inc_small; }
  const T & inc_large() const { return _inc_large; }

  inline virtual void inc_small_action();
  inline virtual void inc_large_action();
  inline virtual void dec_small_action();
  inline virtual void dec_large_action();

  // Widget.
  
  int handle(int);

protected:

  inline void clamp(bool);
  virtual void value_update() {}
  virtual void range_update() {}

private:

  T _value;
  bool _default_valid;
  T _default_value;
  T _min, _max;
  bool _clamp;
  T _inc_small, _inc_large;
};

//------------------------------------------------------------------------------
// Integer Base

class Int_Base : public Base<int>
{
public:

  Int_Base();
};

//------------------------------------------------------------------------------
// Float Base

class Float_Base : public Base<double>
{
public:

  Float_Base();
  virtual ~Float_Base() {}

  // Value.
  
  virtual void precision(int);
  virtual int precision() const { return _precision; }
  
private:

  int _precision;
};

//------------------------------------------------------------------------------
// Increment Tool Widget

class Inc : public widget::Widget
{
public:

  Inc();
  
  Signal<bool> inc_sig;
  Signal<bool> dec_sig;
  
private:

  DJV_CALLBACK(Inc, inc_callback, bool);
  DJV_CALLBACK(Inc, dec_callback, bool);
  
  button::Tool * _inc_button, * _dec_button;
};

}
}
}

#include <djv_ui/num_.h>

#endif

