/*------------------------------------------------------------------------------
 lib/djv_ui/float.h
 
 Floating Point Number Widgets
 -----------------------------

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_FLOAT_H
#define DJV_UI_FLOAT_H

#include <djv_ui/num.h>

namespace djv {
namespace ui {
namespace num
{

//------------------------------------------------------------------------------
// Float Edit

class Float_Edit : public widget::Widget, public Float_Base
{
public:

  Float_Edit();
  virtual ~Float_Edit() {}
  
  // Widget.
  
  virtual int handle(int);
  
  // Layout.
  
  virtual Item * group_align();

private:

  DJV_CALLBACK(Float_Edit, widget_callback, const Str &);
  DJV_CALLBACK(Float_Edit, inc_callback, bool);
  DJV_CALLBACK(Float_Edit, dec_callback, bool);
  
  void value_update();
  
  text::Edit * _widget;
};

//------------------------------------------------------------------------------
// Float Display

class Float_Display : public widget::Widget
{
public:

  Float_Display();
  virtual ~Float_Display() {}
  
  // Value.
  
  void set(double);

  void precision(int);
  int precision() const { return _precision; }
  
  // Widget.
  
  virtual void text_color(const Fl_Color &);
  virtual const Fl_Color & text_color() const
  { return widget::Widget::text_color(); }
  
  virtual void draw();
  
  // Layout.
  
  virtual void dirty();
  
private:

  int _precision;
  
  text::Display * _widget;
};

//------------------------------------------------------------------------------
// Float Slider

class Float_Slider : public widget::Widget, public Float_Base
{
public:

  Float_Slider();
  virtual ~Float_Slider() {}
  
  // Value.
  
  virtual void default_valid(bool);
  bool default_valid() const { return Float_Base::default_valid(); }
  
  virtual void range(double min, double max);
  virtual void slider_range(double min, double max);
  
  // Label.
  
  void label(widget::Widget *);
  widget::Widget * label() const { return _label; }
  virtual Item * label_align() const { return _label; }

  // Widget.
  
  virtual int handle(int);
  
  // Layout.
  
  virtual Item * group_align();
  
private:

  DJV_CALLBACK(Float_Slider, widget_callback, const Str &);
  DJV_CALLBACK(Float_Slider, inc_callback, bool);
  DJV_CALLBACK(Float_Slider, dec_callback, bool);
  DJV_CALLBACK(Float_Slider, slider_callback, double);
  DJV_CALLBACK(Float_Slider, default_callback, bool);
  
  void value_update();
  void range_update();
  
  text::Edit * _widget;
  slider::Float * _slider;
  widget::Widget * _label;
  button::Tool * _default_widget;
  layout::Horizontal * _layout;
};

}
}
}

#endif
