/*------------------------------------------------------------------------------
 lib/djv_ui/prefs.h
 
 Preferences
 -----------

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_PREFS_H
#define DJV_UI_PREFS_H

#include <djv_base/box.h>
#include <djv_image/color.h>
#include <djv_image/time.h>
#include <djv_ui/base.h>

class Fl_Preferences;

namespace djv {
namespace ui {
namespace prefs
{

//------------------------------------------------------------------------------
// Preferences

class Prefs
{
public:

  Prefs(const Str & name);
  Prefs(const Prefs &, const Str & group);
  Prefs(Prefs *, const Str & group);
  ~Prefs();

  void set(const Str &, const List<Str> &);
  bool get(const Str &, List<Str> *) const;
  
  void del(const Str &);
  void del_group(const Str &);
  
  List<Str> list() const;
  List<Str> group_list() const;

private:

  std::auto_ptr<Fl_Preferences> _prefs;
};

static const Str separator = ";";

//------------------------------------------------------------------------------
// Image Preferences

class Image
{
public:

  Image();
  ~Image();

  void speed(const djv::image::time::Speed &);
  const djv::image::time::Speed & speed() const;
};

//------------------------------------------------------------------------------
// Global

Prefs * app();
Prefs * global();

Image * image();

void init();
void del();

}
}
}

#endif
