/*------------------------------------------------------------------------------
 lib/djv_ui/slider.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_ui/slider.cxx"

#include "slider.h"
#include <djv_ui/style.h>
#include <djv_ui/button.h>
#include <FL/Fl.H>
#include <FL/fl_draw.H>

namespace djv {
namespace ui {
namespace slider
{

Base::Base() : _handle_size(11)
{
  Fl_Group::color(FL_BACKGROUND2_COLOR);
  Fl_Group::set_visible_focus();

  frame(FL_DOWN_BOX);

  Item::size(layout::STRETCH, layout::FIXED);
}

int Base::handle(int in)
{
  switch (in)
  {
    case FL_FOCUS:
    case FL_UNFOCUS:
      redraw();
      return 1;
  }
  return Frame::handle(in);
}

void Base::draw()
{
  Frame::draw();
}

void Base::handle_size(int in)
{
  _handle_size = in;
}

void Base::handle_draw(double in, const Box2i & contents)
{
  DJV_DBG_OBJ2(_DBG, "Base::draw");
  DJV_DBG_PRN("in = " << in);
  DJV_DBG_PRN("contents = " << contents);
  
  fl_color(fl_darker(FL_BACKGROUND2_COLOR));
  fl_rectf(contents.x + _handle_size / 2, contents.y + contents.h / 2 - 1,
    contents.w - _handle_size, 2);

  const Box2i box(contents.x + int(in * (contents.w - _handle_size)),
    contents.y, _handle_size, contents.h);
  fl_draw_box(FL_UP_BOX, box.x, box.y, box.w, box.h, FL_BACKGROUND_COLOR);
  if (Fl::focus() == this)
    draw_focus(FL_NO_BOX, box.x + 2, box.y + 2, box.w - 2 * 2, box.h - 2 * 2);
  
  /*fl_color(FL_BLACK);
  const int x =
    contents.x + handle_size / 2 + int(in * (contents.w - _handle_size));
  fl_line(x, contents.y, x, contents.y + contents.h - 1);*/
}

void Base::dirty()
{
  Frame::dirty();

  fl_font(style::global()->font(), style::global()->font_size());
  const int frame_margin = 2;
  size_hint(Vec2i(style::global()->size_button() * 2, fl_height()) +
    style::global()->margin_text() * 2 + frame_margin * 2);
}

Int::Int()
{
  range(0, 100);
}

int Int::handle(int in)
{
  const Box2i & box = frame_geom();
  switch (in)
  { 
    case FL_PUSH:
    case FL_DRAG:
    {
      take_focus();
      const int size = handle_size();
      const double v =
        (Fl::event_x() - box.x - ((size - 1) / 2) +
          ((box.w - size - 1) / double(max() - min()) / 2)) /
        double(box.w - size - 1);
      set(min() + int(v * (max() - min())));
    } return 1;
    
    case FL_KEYBOARD:
      switch (Fl::event_key())
      {
        case FL_Home: set(min()); return 1;
        case FL_End: set(max()); return 1;
        case FL_Left:
        case FL_Down: dec_small_action(); return 1;
        case FL_Right:
        case FL_Up: inc_small_action(); return 1;
        case FL_Page_Down: dec_large_action(); return 1;
        case FL_Page_Up: inc_large_action(); return 1;
      }
      break;
  }
  if (num::Int_Base::handle(in)) return 1;
  return slider::Base::handle(in);
}

void Int::draw()
{
  Frame::draw();
  handle_draw(min() != max() ? ((get() - min()) / double(max() - min())) : 0.0,
    frame_geom());
}

void Int::value_update()
{
  redraw();
}

Float::Float()
{
  range(0.0, 1.0);
}

int Float::handle(int in)
{
  const Box2i & box = frame_geom();
  switch (in)
  {
    case FL_PUSH:
    case FL_DRAG:
    {
      take_focus();
      const int size = handle_size();
      const int w = box.w - size - 1;
      const double v = w ?
        ((Fl::event_x() - box.x - (size - 1) / 2) / double(w)) : 0.0;
      set(min() + v * (max() - min()));
    } return 1;
    
    case FL_KEYBOARD:
      switch (Fl::event_key()) {
        case FL_Home: set(min()); return 1;
        case FL_End: set(max()); return 1;
        case FL_Left:
        case FL_Down: dec_small_action(); return 1;
        case FL_Right:
        case FL_Up: inc_small_action(); return 1;
        case FL_Page_Down: dec_large_action(); return 1;
        case FL_Page_Up: inc_large_action(); return 1;
      }
      break;
  }
  if (num::Float_Base::handle(in)) return 1;
  return slider::Base::handle(in);
}

void Float::draw()
{
  Frame::draw();
  handle_draw(max() != min() ? ((get() - min()) / double(max() - min())) : 0.0,
    frame_geom());
}

void Float::value_update()
{
  redraw();
}

}
}
}
