/*------------------------------------------------------------------------------
 lib/djv_ui/style.h
 
 Style
 -----
 
 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_STYLE_H
#define DJV_UI_STYLE_H

#include <djv_image/color.h>
#include <djv_ui/base.h>

namespace djv {
namespace ui {
namespace style
{

//------------------------------------------------------------------------------
// Options

enum COLOR { COLOR_DARK, COLOR_NEUTRAL, COLOR_LIGHT };
enum SIZE { SIZE_SMALL, SIZE_MEDIUM, SIZE_LARGE };

//------------------------------------------------------------------------------
// Style

class Style
{
public:

  Style();

  // Color.

  void color(COLOR);
  COLOR color() const;
  const djv::image::Color & color_foreground() const;
  const djv::image::Color & color_background() const;
  const djv::image::Color & color_background2() const;
  const djv::image::Color & color_select() const;

  // Size.

  void size(SIZE);
  SIZE size() const;
  int size_pick() const;
  int size_handle() const;

  // Font.

  Fl_Font font() const;
  Fl_Font font_bold() const;
  int font_size() const;
  
  // Miscellaneous.

  void timecode(bool);
  bool timecode() const;

  void tooltips(bool);
  bool tooltips() const;

  // Layout.

  int spacing() const;
  int spacing_large() const;
  
  int margin() const;
  int margin_text() const;
  int margin_widget() const;
  int margin_large() const;

  int size_icon() const;
  int size_tool_icon() const;
  int size_button() const;
  int size_text() const;
  int size_swatch() const;
  int size_thumbnail() const;

private:

  class _Style;
  
  std::auto_ptr<_Style> _p;
};

//------------------------------------------------------------------------------
// Labels

const List<Str> & label_color();
const List<Str> & label_size();

//------------------------------------------------------------------------------
// Global

Style * global();

void del();

}
}
}

#endif

