/*------------------------------------------------------------------------------
 lib/djv_ui/tab.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_ui/tab.cxx"

#include "tab.h"
#include <djv_ui/style.h>
#include <djv_ui/layout_stack.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/scroll.h>
#include <FL/Fl.H>
#include <FL/fl_draw.H>

namespace djv {
namespace ui {
namespace tab
{

class _Widget : public widget::Widget
{
public:

  _Widget(widget::Base *);
  
  void name(const Str &);
  const Str & name() const { return _name; }
  
  Str dbg_name() const { return "djv::ui::tab::_Widget"; }
  
private:

  Str _name;
  mem::Buffer<char> _name_ascii;
};

_Widget::_Widget(widget::Base * widget)
{
  //text_font(style::global()->font_bold());
  
  //layout::Vertical * layout = new layout::Vertical(this);
  //if (widget) layout->add(widget);
  
  scroll::Widget * scroll_widget = new scroll::Widget;
  
  layout::Vertical * layout = new layout::Vertical(this);
  layout->add(scroll_widget);
  scroll_widget->layout()->margin(0);
  if (widget) scroll_widget->layout()->add(widget);
}

void _Widget::name(const Str & in)
{
  if (in == _name) return;
  
  _name = in;
  
  const int size = _name.size() + 1;
  _name_ascii.size(size);
  str::ascii(_name, _name_ascii(), size);
  Fl_Group::label(_name_ascii());
}

Widget::Widget() : Fl_Tabs(0, 0, 0, 0)
{
  Fl_Tabs::end();

  layout::Stack * layout = new layout::Stack(this);
  layout->margin(0);
  
  callback(widget_callback, this);
}

void Widget::set(int in)
{
  DJV_ASSERT(in < int(_list.size()));
  
  value(dynamic_cast<Fl_Widget *>(_list[in]));
}

int Widget::get() const
{
  // XXX FLTK forgot their const.

  Widget * widget = const_cast<Widget *>(this);
  
  for (uint i = 0; i < _list.size(); ++i)
    if (widget->value() == dynamic_cast<Fl_Widget *>(_list[i]))
      return i;
      
  return -1;
}

void Widget::tab_add(const Str & name, widget::Base * in)
{
  DJV_DBG_OBJ2(_DBG, "Widget::tab_add");
  DJV_DBG_PRN("name = " << name);
  
  _Widget * widget = new _Widget(in);
  widget->name(name);
  _list += widget;
  
  layout()->add(widget);
  
  //dirty();
  update();
  redraw();
  
  //tab(_list.size() - 1);
}
  
void Widget::tab_name(const Str & name, int tab)
{
  DJV_ASSERT(tab < int(_list.size()));
  
  DJV_DBG_OBJ2(_DBG, "Widget::tab_name");
  DJV_DBG_PRN("name = " << name);
  DJV_DBG_PRN("tab = " << tab);
  
  static_cast<_Widget *>(_list[tab])->name(name);
  
  //dirty();
  update();
  redraw();
}

void Widget::tab_del(int in)
{
  DJV_ASSERT(in < int(_list.size()));
  
  DJV_DBG_OBJ2(_DBG, "Widget::tab_del");
  DJV_DBG_PRN("in = " << in);

  List<widget::Base *>::iterator i = _list.begin();
  for (int j = 0; j < in; ++j, ++i) ;
  _list.erase(i);
  
  layout()->del(in);
  
  //dirty();
  update();
  redraw();
}

void Widget::tab_del()
{
  while (_list.size()) tab_del(0);
}

int Widget::handle(int in)
{
  switch (widget::Base::handle(in))
  {
    case -1: return 0;
    case 1: return 1;
  }
  return Fl_Tabs::handle(in);
}
  
void Widget::draw()
{
  Fl_Tabs::draw();

  //const Box2i & geom = this->geom();

  // XXX Erase extra tab lines.
  
  /*fl_color(FL_BACKGROUND_COLOR);
  fl_line(geom.x, geom.y + _size_tab.y, geom.x, geom.y + geom.h - 1);
  fl_line(geom.x + geom.w - 1, geom.y + _size_tab.y, geom.x + geom.w - 1,
    geom.y + geom.h - 1);
  fl_line(geom.x, geom.y + geom.h - 1, geom.x + geom.w - 1,
    geom.y + geom.h - 1);*/
  
  widget::Base::draw();
}

void Widget::dirty()
{
  DJV_DBG_OBJ2(_DBG, "Widget::dirty");
  
  widget::Base::dirty();

  fl_font(text_font(), style::global()->font_size());
  int label_hint = 0;
  for (uint i = 0; i < _list.size(); ++i)
    label_hint = math::max(
      str_width(static_cast<_Widget *>(_list[i])->name()),
      label_hint);

  _size_tab = Vec2i(0, fl_height()) + style::global()->margin_widget() * 2 +
    Vec2i(1 * 2, 1);
  DJV_DBG_PRN("tab = " << _size_tab);

  size_hint(_size_tab + layout()->size_hint() + 1 * 2);
}

void Widget::geom(const Box2i & in)
{
  DJV_DBG_OBJ2(_DBG, "Widget::geom");
  DJV_DBG_PRN("in = " << in);
  
  Fl_Widget::resize(in.x, in.y, in.w, in.h);
  layout::Item::geom(in);
  layout()->geom(Box2i(in.x + 1, in.y + _size_tab.y + 1, in.w - 1 * 2,
    in.h - _size_tab.y - 1 * 2));
}

void Widget::widget_callback()
{
  sig.emit(get());
}

}
}
}
