/*------------------------------------------------------------------------------
 plugin/djv_quicktime_apple/fnc.cxx
 
 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "plugin/djv_quicktime_apple/fnc.cxx"

#include "plugin.h"
#include <QTML.h>
#include <Movies.h>

namespace { int _ref = 0; }

void quicktime_init() throw (Error)
{
  ++_ref;
  if (_ref > 1) return;

  DJV_DBG_OBJ2(_DBG, "quicktime_init");

  OSErr err = InitializeQTML(kInitializeQTMLNoSoundFlag);
  if (err != noErr)
    throw Error(name, str::Format(error_init).arg(err));
  
  long version;
  err = Gestalt(gestaltQuickTime, &version);
  if (err != noErr)
    throw Error(name, str::Format(error_version).arg(err));
  
  err = EnterMovies();
  if (err != noErr)
    throw Error(name, str::Format(error_init).arg(err));
}

void quicktime_del()
{
  --_ref;
  if (_ref) return;
  
  DJV_DBG_OBJ2(_DBG, "quicktime_del");

  ExitMovies();
  TerminateQTML();
}

CodecType quicktime_codec(CODEC in)
{
  switch (in)
  {
    case CODEC_RAW: return kRawCodecType;
    case CODEC_JPEG: return kJPEGCodecType;
    case CODEC_MJPEGA: return kMotionJPEGACodecType;
    case CODEC_MJPEGB: return kMotionJPEGBCodecType;
    case CODEC_H263: return kH263CodecType;
    case CODEC_H264: return kH264CodecType;
    case CODEC_DVCNTSC: return kDVCNTSCCodecType;
    case CODEC_DVCPAL: return kDVCPALCodecType;
  }
  return 0;
}

CodecQ quicktime_quality(QUALITY in)
{
  switch (in)
  {
    case QUALITY_LOSSLESS: return codecLosslessQuality;
    case QUALITY_MIN: return codecMinQuality;
    case QUALITY_MAX: return codecMaxQuality;
    case QUALITY_LOW: return codecLowQuality;
    case QUALITY_NORMAL: return codecNormalQuality;
    case QUALITY_HIGH: return codecHighQuality;
  }
  return 0;
}

const List<Str> & label_codec()
{
  static const List<Str> data = List<Str>() << "Raw" << "JPEG" << "MJPEG-A" <<
    "MJPEG-B" << "H263" << "H264" << "DVC-NTSC" << "DVC-PAL";
  return data;
}

const List<Str> & label_quality()
{
  static const List<Str> data = List<Str>() << "Lossless" << "Min" << "Max" <<
    "Low" << "Normal" << "High";
  return data;
}

const List<Str> & label_save_option()
{
  static const List<Str> data = List<Str>() << "Codec" << "Quality";
  return data;
}

