/*------------------------------------------------------------------------------
 plugin/djv_quicktime_unix/plugin.h
 
 UNIX QuickTime Image I/O
 ------------------------
 
 Requires:
 * libquicktime - http://libquicktime.sourceforge.net

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_QUICKTIME_UNIX_PLUGIN_H
#define DJV_QUICKTIME_UNIX_PLUGIN_H

#include <djv_image/io.h>
#include <lqt.h>
#include <colormodels.h>

using namespace djv;

//------------------------------------------------------------------------------
// Plugin Information

static const Str name = "QuickTime";

static const List<Str> extension_list = List<Str>() << ".qt" << ".mov"
  << ".avi";

//------------------------------------------------------------------------------
// Load Plugin

class Load : public image::io::Load
{
public:

  Load();
  ~Load();

  plugin::Plugin * copy() const { return new Load; }
  
  Str name() const { return ::name; }
  const List<Str> & extension_list() const { return ::extension_list; }
  
  void open(const File &, image::Info *, image::Tag *, image::Time *)
    throw (Error);
  void open(const File & in, List<image::Info> * info, image::Tag * tag,
    image::Time * time) throw (Error)
  { image::io::Load::open(in, info, tag, time); }
  const image::Image * get(int frame, int layer, int proxy) throw (Error);

private:

  void _close();
  
  File _file;
  image::Info _info;
  image::Time _time;
  int _frame;
  quicktime_t * _f;
  image::Image _image;
  image::Data _tmp;
};

//------------------------------------------------------------------------------
// Save Plugin

class Save : public image::io::Save
{
public:

  Save();
  
  plugin::Plugin * copy() const;
  
  Str name() const { return ::name; }
  const List<Str> & extension_list() const { return ::extension_list; }
  Str help() const;
  
  struct Options
  {
    Options() : codec("jpeg") {}
    Str codec;
  };
  enum OPTION { OPTION_CODEC };
  bool option(const Str &, List<Str> *);
  List<Str> option(const Str &) const;
  const List<Str> & option_list() const;
  
  void open(const File &, const image::Info &, const image::Tag &,
    const image::Time &) throw (Error);
  void close() throw (Error);
  void set(const image::Image &, int frame) throw (Error);

private:
  
  Options _options;
  File _file;
  quicktime_t * _f;
  int _frame_duration;
  image::Info _info;
  image::Image _image;
};

//------------------------------------------------------------------------------
// Labels

static const Str
  error_video = "No Video Tracks",
  error_codec = "Codec",
  error_colormodel = "Color Model";

List<Str> label_codec();
const List<Str> & label_save_option();

#endif

