/*------------------------------------------------------------------------------
 plugin/djv_tiff/plugin.h

 Tagged Image File Format (TIFF) I/O
 -----------------------------------

 Requires: 
 * libtiff - http://www.libtiff.org
 
 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_TIFF_PLUGIN_H
#define DJV_TIFF_PLUGIN_H

#include <djv_image/io.h>
#include <tiffio.h>

using namespace djv;

//------------------------------------------------------------------------------
// Plugin Information

static const Str name = "TIFF";

static const List<Str> extension_list = List<Str>() << ".tiff" << ".tif";

//------------------------------------------------------------------------------
// Plugin Options

enum COMPRESS { COMPRESS_NONE, COMPRESS_RLE, COMPRESS_LZW };

//------------------------------------------------------------------------------
// Load Plugin

class Load : public image::io::Load
{
public:

  Load();
  ~Load();

  plugin::Plugin * copy() const { return new Load; }
  
  Str name() const { return ::name; }
  const List<Str> & extension_list() const { return ::extension_list; }
  
  void open(const File &, image::Info *, image::Tag *, image::Time *)
    throw (Error);
  void open(const File & in, List<image::Info> * info, image::Tag * tag,
    image::Time * time) throw (Error)
  { image::io::Load::open(in, info, tag, time); }
  const image::Image * get(int frame, int layer, int proxy) throw (Error);

private:

  void _open(const File &, image::Info *, image::Tag *) throw (Error);
  void _close();

  File _file;
  TIFF * _f;
  bool _planar, _compress, _palette;
  uint16 * _colormap [3];
  image::Image _image;
  image::Data _tmp;
};

//------------------------------------------------------------------------------
// Save Plugin

class Save : public image::io::Save {
public:

  Save();
  ~Save();
  
  plugin::Plugin * copy() const;
  
  Str name() const { return ::name; }
  const List<Str> & extension_list() const { return ::extension_list; }
  Str help() const;
  
  struct Options
  {
    Options() : compress(COMPRESS_NONE) {}
    COMPRESS compress;
  };
  enum OPTION { OPTION_COMPRESS };
  bool option(const Str &, List<Str> *);
  List<Str> option(const Str &) const;
  const List<Str> & option_list() const;
  
  void open(const File &, const image::Info &, const image::Tag &,
    const image::Time &) throw (Error);
  void set(const image::Image &, int frame) throw (Error);

private:

  void _open(const File &, const image::Tag &) throw (Error);
  void _close();

  Options _options;
  File _file;
  TIFF * _f;
  image::Info _info;
  image::Image _image;
};

//------------------------------------------------------------------------------
// Functions

void tiff_init();

void palette_unpack(uint8_t *, int size, int bytes,
  uint16_t * red, uint16_t * green, uint16_t * blue);

//------------------------------------------------------------------------------
// Labels

const List<Str> & label_compress();
const List<Str> & label_save_option();

#endif

