//#define DJV_DBG

#include <djv_ui/layout_row.h>
#include <djv_ui/window.h>
#include <djv_ui/button.h>
#include <djv_ui/app.h>

namespace
{
using namespace djv;

class App : public ui::App
{
public:

  App(int & argc, char ** argv);
  ~App();

private:

  DJV_CALLBACK(App, push_callback, bool) { print("push"); }
  
  DJV_CALLBACK(App, check_callback, bool)
    { print(str::Format("check = %1").arg(in)); }
  
  DJV_CALLBACK(App, radio_callback0, bool) { print("radio 0"); }
  DJV_CALLBACK(App, radio_callback1, bool) { print("radio 1"); }
  DJV_CALLBACK(App, radio_callback2, bool) { print("radio 2"); }
  
  DJV_CALLBACK(App, tool_callback0, bool) { print("tool"); }
  DJV_CALLBACK(App, tool_callback1, bool)
    { print(str::Format("tool = %1").arg(in)); }
  DJV_CALLBACK(App, tool_callback2, bool) { print("tool repeat"); }
  
  DJV_CALLBACK(App, exit_callback, bool) { exit(); }

  ui::window::Widget * _window;
};

App::App(int & argc, char ** argv) :
  ui::App("buttons", argc, argv),
  _window(0)
{
  DJV_DBG_OBJ("App::App");
  
  // Widgets.
  
  ui::button::Push * push_button = new ui::button::Push("&Push Button");
  
  ui::button::Check * check_button = new ui::button::Check("&Check Button");
  
  ui::button::Radio * radio_button0 = new ui::button::Radio("Radio &1");
  ui::button::Radio * radio_button1 = new ui::button::Radio("Radio &2");
  ui::button::Radio * radio_button2 = new ui::button::Radio("Radio &3");
  ui::button::radio_group(List<ui::button::Radio *>() << radio_button0 <<
    radio_button1 << radio_button2);
  
  ui::button::Tool * tool_button0 = new ui::button::Tool("file");
  tool_button0->type(ui::button::Tool::PUSH);
  ui::button::Tool * tool_button1 = new ui::button::Tool("dir");
  tool_button1->type(ui::button::Tool::TOGGLE);
  ui::button::Tool * tool_button2 = new ui::button::Tool("magnify");
  tool_button2->type(ui::button::Tool::REPEAT);

  _window = new ui::window::Widget(name());

  // Layout.
  
  ui::layout::Vertical * layout = new ui::layout::Vertical(_window);
  layout->add(push_button);
  layout->add(check_button);
  
  ui::layout::Vertical * v_layout = new ui::layout::Vertical(layout);
  v_layout->margin(0);
  v_layout->add(radio_button0);
  v_layout->add(radio_button1);
  v_layout->add(radio_button2);

  ui::layout::Horizontal * h_layout = new ui::layout::Horizontal(layout);
  h_layout->margin(0);
  h_layout->add(tool_button0);
  h_layout->add(tool_button1);
  h_layout->add(tool_button2);
  
  layout->add_stretch();
  
  // Initialize.
  
  radio_button0->set(true);
  
  _window->size(_window->size_hint());
  _window->show();

  // Callbacks.
  
  push_button->sig.set(this, push_callback);
  check_button->sig.set(this, check_callback);
  radio_button0->sig.set(this, radio_callback0);
  radio_button1->sig.set(this, radio_callback1);
  radio_button2->sig.set(this, radio_callback2);
  tool_button0->sig.set(this, tool_callback0);
  tool_button1->sig.set(this, tool_callback1);
  tool_button2->sig.set(this, tool_callback2);
  _window->close_sig.set(this, exit_callback);

  valid(true);
}

App::~App()
{
  if (_window) _window->widget_del();
}

}

int main(int argc, char ** argv) { return ::App(argc, argv).exec(); }
