//#define DJV_DBG

#include <djv_base/math.h>
#include <djv_image/color.h>
#include <djv_image/data.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/window.h>
#include <djv_ui/group.h>
#include <djv_ui/label.h>
#include <djv_ui/button.h>
#include <djv_ui/tab.h>
#include <djv_ui/color.h>
#include <djv_ui/app.h>
#include <FL/Fl_Image.H>

namespace
{
using namespace djv;

class App : public ui::App
{
public:

  App(int & argc, char ** argv);
  ~App();

private:

  void add();
  void del();
  
  DJV_CALLBACK(App, widget_callback, int)
    { print(str::Format("tab = %1").arg(in)); }
  DJV_CALLBACK(App, add_callback, bool) { add(); }
  DJV_CALLBACK(App, del_callback, bool) { del(); }
  DJV_CALLBACK(App, exit_callback, bool) { exit(); }

  ui::tab::Widget * _widget;
  ui::window::Widget * _window;
};

App::App(int & argc, char ** argv) :
  ui::App("tab", argc, argv),
  _window(0)
{
  // Widgets.
   
  _widget = new ui::tab::Widget;

  ui::button::Tool * add_button = new ui::button::Tool("tool_add");
  add_button->type(ui::button::Tool::REPEAT);
  ui::button::Tool * del_button = new ui::button::Tool("tool_x");
  del_button->type(ui::button::Tool::REPEAT);

  ui::group::Frame * frame = new ui::group::Frame;
  frame->edges(false, false, true, false);

  _window = new ui::window::Widget(name());
  
  // Layout.
  
  ui::layout::Vertical * layout = new ui::layout::Vertical(_window);
  layout->spacing(0);
  layout->margin(0);
  layout->add(_widget);
  layout->stretch(_widget);
  
  layout->add(frame);
  ui::layout::Horizontal * h_layout = new ui::layout::Horizontal(frame);
  h_layout->add_stretch();
  h_layout->add(add_button);
  h_layout->add(del_button);
  
  // Initialize.
  
  add();
  
  _window->size(Vec2i(300, 300));
  _window->show();
  
  // Callbacks.
  
  _widget->sig.set(this, widget_callback);
  add_button->sig.set(this, add_callback);
  del_button->sig.set(this, del_callback);
  _window->close_sig.set(this, exit_callback);

  valid(true);
}

App::~App()
{
  if (_window) _window->widget_del();
}

void App::add()
{
  static const Str data [] = { "stingray", "striped bass", "leopard shark" };

  //_widget->tab_add(str::Format("tab %1").arg(_widget->tab_size()),
  //  new ui::label::Widget(data[0]));

  _widget->tab_add(str::Format("tab %1").arg(_widget->tab_size()),
    new ui::color::Widget);

  /*List<Str> text;
  const int size = math::floor(math::rand(20, 50));
  for (uint i = 0; i < size; ++i)
    text += data[math::floor(math::rand(2.999999))];
  
  _widget->tab_add(str::Format("tab %1").arg(_widget->tab_size()),
    new ui::label::Text(str::join(text, " ")));*/
}

void App::del()
{
  if (! _widget->tab_size()) return;
  _widget->tab_del(_widget->get());
}

}

int main(int argc, char ** argv) { return ::App(argc, argv).exec(); }
